/*
!C******************************************************************

!Description: 
		meta.c



!Input Parameters:
!Output Parameters: 
!Revision History:
!Team-unique Header:
!References and Credits 
!Design Notes


DESCRIPTION


!END*************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "meta.h"
#include "struct.h"
#include "protos.h"
#include "qa_map.h"
#include "versions.h"

#define CLEARFILL(x)  ((x) == -1 ? 0 : (x))

/* global error buffer */
static char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */

/*  global tracker that determines when to allocate and
   free input_metadata structure */
static int32 input_metadata_inuse=0;

int32 shrink_Mstringarray(marray_t *string);
void grow_Mstringarray(marray_t *string, int32 length, char *value);

void put_qa_into_metadata(int bands[], qavals_t *qavalues, input_metadata *metadata, int nline_1km)
{
#define YES "yes"
#define NO  "no"
void initqavalues(qavals_t *qavalues);
int i, j, ich, rez_processed; 
long totalpixels=0L;
long aerototalpixels=0L;

      if(commands.process_sea)strcpy(metadata->seaprocessed, YES);
      else strcpy(metadata->seaprocessed, NO);

      if(commands.adja_on)strcpy(metadata->adjacencycorrectionattempted, YES);
      else strcpy(metadata->adjacencycorrectionattempted, NO);

      if(commands.ref_lam)strcpy(metadata->atmosphericcorrectionattempted, YES);
      else strcpy(metadata->atmosphericcorrectionattempted, NO);

/*      if(commands.brdf_mode==BRDF_MONTANA)strcpy(metadata->brdfcorrectionattempted, "montana");
      else if(commands.brdf_mode==BRDF_BOSTON)strcpy(metadata->brdfcorrectionattempted, "boston");*/
 
      /* calculate band quality metadata */
      for(ich=0;ich<BANDPROCESS;ich++){
	if(!bands[ich])continue;
	if(ich<BAND1_500M){
	  for(i=0;i<16;i++)metadata->qualityclasspercentage[ich][i]=(int32)
			     (((float32)qavalues->quality_count[ich][i]/(float32)(NBSPOTS_250M*nline_1km*4))*100+0.5F);
	  totalpixels=totalpixels+(NBSPOTS_250M*nline_1km*4);
	}
	else if(ich<BAND1_1KM){
	  for(i=0;i<16;i++)metadata->qualityclasspercentage[ich][i]=(int32)
	    (((float32)qavalues->quality_count[ich][i]/(float32)(NBSPOTS_500M*nline_1km*2))*100+0.5F);
	  totalpixels=totalpixels+(NBSPOTS_500M*nline_1km*2);
	}
	else {
	  for(i=0;i<16;i++)metadata->qualityclasspercentage[ich][i]=(int32)
	    (((float32)qavalues->quality_count[ich][i]/(float32)(NBSPOTS_1KM*nline_1km))*100+0.5F);
	  totalpixels=totalpixels+(NBSPOTS_1KM*nline_1km);
	}
      }/* for(ich... */

      /* store the % of pixels atmospherically corrected */
      for(ich=0;ich<BANDPROCESS;ich++){
	if(!bands[ich])continue;
	metadata->qualityclasspercentage[ich][8]=(int32)((((float64)qavalues->atmcor_percentage)/((float64)totalpixels))*100+0.5);
      }
      /* calculate MODLAND ECS metadata for reflectance */
      metadata->qapercentgoodquality=(int32)((((float64)qavalues->qapercentgoodquality)/((float64)totalpixels))*100+0.5);
      metadata->qapercentother=(int32)((((float64)qavalues->qapercentother)/((float64)totalpixels))*100+0.5);
      metadata->qapercentnotproducedcloud=(int32)((((float64)qavalues->qapercentnotproducedcloud)/((float64)totalpixels))*100+0.5);
      metadata->qapercentnotproducedother=(int32)((((float64)qavalues->qapercentnotproducedother)/((float64)totalpixels))*100+0.5);
      metadata->percentnotsea=(int32)((((float64)qavalues->percentnotsea)/((float64)totalpixels))*100+0.5);

      for(i=0;i<3;i++) {
          rez_processed = n_bands_res[i];
          switch (i) {
	      case 0:
                 if (rez_processed > 0) aerototalpixels += (NBSPOTS_250M*nline_1km*4);
		 break;
	      case 1:
                 if (rez_processed > 0) aerototalpixels += (NBSPOTS_500M*nline_1km*2);
		 break;
	      case 2:
                 if (rez_processed > 0) aerototalpixels += (NBSPOTS_1KM*nline_1km);
		 break;
		 }
	  }
     
      for(i=0;i<4;i++){
	metadata->aerosol_clmh[i]=
	  (PGSt_integer)((((float64)qavalues->aerosol_clmh[i])/((float64)aerototalpixels))*100+0.5);
           }

      /* calculate mandatory ECS metadata for reflectance */
      metadata->qapercentoutofbounds=(int32)((((float64)qavalues->qapercentoutofbounds)/((float64)totalpixels))*100+0.5);
      metadata->qapercentmissing=(int32)((((float64)qavalues->qapercentmissing)/((float64)totalpixels))*100+0.5);
      metadata->highsolarzenith=qavalues->highsolarzenith;


      /* CALCULATE AUTOMATICQUALITY FLAG for MOD09 */
      if(metadata->qapercentoutofbounds + metadata->qapercentmissing >= 100)
         metadata->AutoQualityIsGood=AUTOBAD;
      if(metadata->qapercentnotproducedcloud + metadata->qapercentnotproducedother >= 100)
         metadata->AutoQualityIsGood=AUTOBAD;
      if(metadata->percentnotsea==0)
         metadata->AutoQualityIsGood=AUTOBAD;
      if(100*metadata->highsolarzenith/(nline_1km*NBSPOTS_1KM) > 50)  /* new, 25-JUN-99, Jim Ray */
         metadata->AutoQualityIsGood=AUTOSUSPECT;

      if(metadata->AutoQualityIsGood == AUTOGOOD)
         strcpy(metadata->autoquality, "Passed");
      else if (metadata->AutoQualityIsGood == AUTOSUSPECT)
         strcpy(metadata->autoquality, "Suspect");
      else 
         strcpy(metadata->autoquality, "Failed");

      
      initqavalues(qavalues);
}

void writeQAattributes(dims_struct *outputs, int sds_index)
{
int32 sds;
char message[3000] = {"\0"};
char location[50]="meta.c, writeQAattributes\0";

sds = outputs->sdsid[sds_index];
if (sds != -1) {
   switch (sds_index) {
      case AOTCM: 
          strcpy(message, QAMAP_INTCM);   /* see "qa_map.h" */
          break;
      case AOTQA: 
          strcpy(message, QAMAP_AOTQA);   /* see "qa_map.h" */
          break;
      case QKMQA:
          strcpy(message, QAMAP_250M);   
          break;
      case HKMQA:
          strcpy(message, QAMAP_500M);   
          break;
      case OKMQA:
          strcpy(message, QAMAP_1KM);   
          break;
      default:
          sprintf(msg, "QA index for this SDS is undefined (bug in code), line %d, file %s", __LINE__, __FILE__);
          modsmf(MODIS_E_LHDF_ERROR, msg, location);
          break;
      }
      
   if(SDsetattr(sds, "QA index\0", DFNT_CHAR, strlen(message), (void *)message)){
       sprintf(msg, "Unable to write QA map indices for QA");
       modsmf(MODIS_E_LHDF_ERROR, msg, location);
     }   


  }

}


void readwriteStateQAattributes(int32 stateqa_sds, int32 fine_sdid, int32 coarse_sdid, long n_lines)
/*
!C*****************************************************************************
!Description: reads the State QA band, writes statistics based upon the State QA
              as HDF attributes to the State QA SDS.

!Input Parameters:
stateqa_sds: state qa SDS handle
coarse_sdid: coarse file SD handle
n_lines: the number of lines to read from the State QA

!Output Parameters: 
none

!Input/Output Parameters: none

!Returns: nothing

!Revision History:
	Original version 01-JUN-99, J. Ray (jim@cratmos.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Jim Ray
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		jim@cratmos.gsfc.nasa.gov, (301) 614-6684

!Design Notes
     
!END***************************************************************************
*/
{
char location[50]="meta.c, readwriteStateQAattributes\0";
char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */
int iarray, iparam;	/* parameter and array indices */
char message[3000] = {"\0"};
char state_stats[1500]={'\0'};
uint16 *state_qa;
int32 start[2], dims[2];
long li, n_pixels;
uint16 value, IntSno, BRDF, Adj, SI, IntFire, IntCld, Cirr, Aer, LandSea, CldSh, Cld, rets;
float top;
float ISnow, NoBRDF, AdjCld, SnoIce, IFire, ICld, NoCirr, SmCirr, AvgCirr, HiCirr,
      ClimAer, LowAer, AvgAer, HiAer, ShalOc, Land, Coast, ShalIn, Eph, DeepIn, 
      ContOc, DeepOc, CldShd, Clear, Cloudy, Mixed, NtSet;

/* read the state qa data */
n_pixels = n_lines*NBSPOTS_1KM*NBSCANS_1KM;
if ((state_qa = (uint16 *)malloc(n_pixels*sizeof(uint16))) == NULL) {
    sprintf(msg, "unable to allocate memory to read State QA data");
    modsmf(MODIS_E_MEM_FAILURE, location, msg);
    strcpy(message, STATEQAMAP);   /* see "qa_map.h" */
     }
else {
     start[0] = start[1] = 0;
     dims[0] = (int32)n_lines*NBSCANS_1KM;
     dims[1] = (int32)NBSPOTS_1KM;
     if (SDreaddata(stateqa_sds, start, NULL, dims, state_qa) == FAIL) {
         sprintf(msg, "Unable to read State QA for the State QA map indices");
         modsmf(MODIS_E_HDF_FAILURE, msg, location);
         strcpy(message, STATEQAMAP);
         } 
     else {

       /* get total number of pixels that fall into each category */
       ISnow=NoBRDF=AdjCld=SnoIce=IFire=ICld=NoCirr=SmCirr=AvgCirr=HiCirr=0.0F;
       ClimAer=LowAer=AvgAer=HiAer=ShalOc=Land=Coast=ShalIn=Eph=DeepIn=0.0F;
       ContOc=DeepOc=CldShd=Clear=Cloudy=Mixed=NtSet=0.0F;
       
       for (li=0L;li<n_pixels;li++) {
           value   = state_qa[li];
	   IntSno  = (value&32768)>>15;
	   BRDF    = (value&16384)>>14;
	   Adj     = (value&8192)>>13;
	   SI      = (value&4096)>>12;
	   IntFire = (value&2048)>>11;
	   IntCld  = (value&1024)>>10;
	   Cirr    = (value&768)>>8;
	   Aer     = (value&192)>>6;
	   LandSea = (value&56)>>3;
	   CldSh   = (value&4)>>2;
	   Cld     = (value&3);
	   if (IntSno) ISnow++;
	   if (BRDF == 0) NoBRDF++;
	   if (Adj == 1) AdjCld++;
	   if (SI) SnoIce++;
	   if (IntFire) IFire++;
	   if (IntCld) ICld++;
	   if (Cirr == 0) NoCirr++; 
	   if (Cirr == 1) SmCirr++; 
	   if (Cirr == 2) AvgCirr++; 
	   if (Cirr == 3) HiCirr++; 
	   if (Aer == 0) ClimAer++; 
	   if (Aer == 1) LowAer++; 
	   if (Aer == 2) AvgAer++; 
	   if (Aer == 3) HiAer++; 
	   if (LandSea == 0) ShalOc++;
	   if (LandSea == 1) Land++;
	   if (LandSea == 2) Coast++;
	   if (LandSea == 3) ShalIn++;
	   if (LandSea == 4) Eph++;
	   if (LandSea == 5) DeepIn++;
	   if (LandSea == 6) ContOc++;
	   if (LandSea == 7) DeepOc++;
	   if (CldSh) CldShd++;
	   if (Cld == 0) Clear++;
	   if (Cld == 1) Cloudy++;
	   if (Cld == 2) Mixed++;
	   if (Cld == 3) NtSet++;
          }
       free(state_qa);
       
       /* Turn into percentages */
       top = 100.0F/((float)n_pixels);
       ISnow *= top;
       NoBRDF *= top;
       AdjCld *= top;
       SnoIce *= top;
       IFire *= top;
       ICld *= top;
       NoCirr *= top;
       SmCirr *= top;
       AvgCirr *= top;
       HiCirr *= top;
       ClimAer *= top;
       LowAer *= top;
       AvgAer *= top;
       HiAer *= top;
       ShalOc *= top;
       Land *= top;
       Coast *= top;
       ShalIn *= top;
       Eph *= top;
       DeepIn *= top;
       ContOc *= top;
       DeepOc *= top;
       CldShd *= top;
       Clear *= top;
       Cloudy *= top;
       Mixed *= top;
       NtSet *= top;
       
       /* Put these values into state qa map (see "qa_map.h") */
       sprintf(message, STATEQAMAP_DELUXE, ISnow, 100.0-ISnow, 100.0-NoBRDF, NoBRDF, AdjCld, 100.0-AdjCld, SnoIce, 
               100.0-SnoIce, IFire, 100.0-IFire, ICld, 100.0-ICld, NoCirr, SmCirr, AvgCirr, HiCirr, ClimAer, 
	       LowAer, AvgAer, HiAer, ShalOc, Land, Coast, ShalIn, Eph, DeepIn, ContOc, DeepOc, 
	       CldShd, 100.0-CldShd, Clear, Cloudy, Mixed, NtSet);
        
       /* Also put these numbers into the global attributes (see "qa_map.h") */ 
       sprintf(state_stats, STATE_STATS_FOR_GLOBAL_METADATA, (int16)(ISnow+0.5), 100-(int16)(NoBRDF+0.5), (int16)(AdjCld+0.5), 
               (int16)(SnoIce+0.5), (int16)(IFire+0.5), (int16)(ICld+0.5), (int16)(NoCirr+0.5), 
	       (int16)(SmCirr+0.5), (int16)(AvgCirr+0.5), (int16)(HiCirr+0.5), (int16)(ClimAer+0.5), (int16)(LowAer+0.5), 
	       (int16)(AvgAer+0.5), (int16)(HiAer+0.5), (int16)(ShalOc+0.5), (int16)(Land+0.5), (int16)(Coast+0.5), 
	       (int16)(ShalIn+0.5), (int16)(Eph+0.5), (int16)(DeepIn+0.5), (int16)(ContOc+0.5), (int16)(DeepOc+0.5),
	       (int16)(CldShd+0.5), (int16)(Clear+0.5), (int16)(Cloudy+0.5), (int16)(Mixed+0.5), (int16)(NtSet+0.5));
               
#ifndef SUBSET
       if (SDsetattr(coarse_sdid, "Percentages of pixels in each State QA state\0", DFNT_CHAR, strlen(state_stats), (void *)state_stats))  {
         sprintf(msg, "Error writing State QA-derived percentages as global attributes, coarse-resolution file");
         modsmf(MODIS_E_LHDF_ERROR, msg, location);
                  }         
#endif
       if (SDsetattr(fine_sdid, "Percentages of pixels in each State QA state\0", DFNT_CHAR, strlen(state_stats), (void *)state_stats))  {
         sprintf(msg, "Error writing State QA-derived percentages as global attributes, full-resolution file");
         modsmf(MODIS_E_LHDF_ERROR, msg, location);
                  }         
          }   
     } 
           
if(SDsetattr(stateqa_sds, "QA index\0", DFNT_CHAR, strlen(message), (void *)message)){
    sprintf(msg, "Unable to write QA map indices for the State QA");
    modsmf(MODIS_E_LHDF_ERROR, msg, location);
    return;
     }   
}



int process_metadata(int bands[], qavals_t *qavalues, dims_struct **outputs, dims_struct *rad1km_input, dims_struct *geo_input, input_metadata *metadata, int n_scans_corrected)
{
char location[50]="meta.c, process_metadata\0";
int i, j, ich, nline_1km; 
void put_qa_into_metadata(int bands[], qavals_t *qavalues, input_metadata *metadata, int lines);


        nline_1km = outputs[0]->nblines;

        /* Set all qa-related parts of the metadata */
        put_qa_into_metadata(bands, qavalues, metadata, nline_1km);

        /* And copy them from the generic metadata to the reflectance outputs' metadata */
        for(i=0;i<BANDPROCESS;i++) 
          for(j=0;j<16;j++) {
             outputs[0]->xxxxx->qualityclasspercentage[i][j] = metadata->qualityclasspercentage[i][j];
             outputs[1]->xxxxx->qualityclasspercentage[i][j] = metadata->qualityclasspercentage[i][j];
	     }
        for(j=0;j<4;j++) {
          outputs[0]->xxxxx->aerosol_clmh[j] =  outputs[1]->xxxxx->aerosol_clmh[j] = metadata->aerosol_clmh[j];
	     }

        outputs[0]->xxxxx->qapercentgoodquality = outputs[1]->xxxxx->qapercentgoodquality = metadata->qapercentgoodquality;
        outputs[0]->xxxxx->qapercentother = outputs[1]->xxxxx->qapercentother = metadata->qapercentother;
        outputs[0]->xxxxx->qapercentnotproducedcloud = outputs[1]->xxxxx->qapercentnotproducedcloud = metadata->qapercentnotproducedcloud;
        outputs[0]->xxxxx->qapercentnotproducedother = outputs[1]->xxxxx->qapercentnotproducedother = metadata->qapercentnotproducedother;
        outputs[0]->xxxxx->percentnotsea = outputs[1]->xxxxx->percentnotsea = metadata->percentnotsea;
        outputs[0]->xxxxx->qapercentoutofbounds = outputs[1]->xxxxx->qapercentoutofbounds = metadata->qapercentoutofbounds;
        outputs[0]->xxxxx->qapercentmissing = outputs[1]->xxxxx->qapercentmissing = metadata->qapercentmissing;
        outputs[0]->xxxxx->highsolarzenith = outputs[1]->xxxxx->highsolarzenith = metadata->highsolarzenith;

        strcpy(outputs[0]->xxxxx->seaprocessed, metadata->seaprocessed);
        strcpy(outputs[1]->xxxxx->seaprocessed, metadata->seaprocessed);

        strcpy(outputs[0]->xxxxx->atmosphericcorrectionattempted, metadata->atmosphericcorrectionattempted);
        strcpy(outputs[1]->xxxxx->atmosphericcorrectionattempted, metadata->atmosphericcorrectionattempted);

        strcpy(outputs[0]->xxxxx->adjacencycorrectionattempted, metadata->adjacencycorrectionattempted);
        strcpy(outputs[1]->xxxxx->adjacencycorrectionattempted, metadata->adjacencycorrectionattempted);

        strcpy(outputs[0]->xxxxx->autoquality, metadata->autoquality);
        strcpy(outputs[1]->xxxxx->autoquality, metadata->autoquality);

  
        /* This copies INPUTPOINTERS. */
        cpy_inputpointer(outputs[0]->xxxxx, metadata);
        cpy_inputpointer(outputs[1]->xxxxx, metadata);

        /* Copy over other RELEVANT metadata (GRINGs, etc).  *NOT* stuff like INPUTPOINTERS. */
        ECSmetadatacopy(outputs[0]->xxxxx, rad1km_input->xxxxx);
        ECSmetadatacopy(outputs[1]->xxxxx, rad1km_input->xxxxx);
		
        /* Make some localgranuleids. */
        strcpy(outputs[0]->xxxxx->localgranuleid, create_localgranuleid(outputs[0]->xxxxx->rbdt, VersionID, REF_ESDTname)); 
        strcpy(outputs[1]->xxxxx->localgranuleidcoarse, create_localgranuleid(outputs[1]->xxxxx->rbdt, VersionID, CRF_ESDTname)); 

        /* and copy this to the global metadata for the CMG, just in case it's necessary */
        ECScpy(metadata, outputs[1]->xxxxx, 0); 

	metadata->granulenumber = geo_input->xxxxx->granulenumber;
	metadata->sci_abnorm    = geo_input->xxxxx->sci_abnorm;
	metadata->sci_state     = geo_input->xxxxx->sci_state;
	metadata->orbitnumber   = geo_input->xxxxx->orbitnumber;
	metadata->instrumentscansprocessed = (int32)n_scans_corrected;

        /* Copy the sci_state, sci_abnorm, granulenumber. */
	outputs[0]->xxxxx->sci_abnorm    = geo_input->xxxxx->sci_abnorm;
	outputs[1]->xxxxx->sci_abnorm    = geo_input->xxxxx->sci_abnorm;
	outputs[0]->xxxxx->sci_state     = geo_input->xxxxx->sci_state;
	outputs[1]->xxxxx->sci_state     = geo_input->xxxxx->sci_state;
	outputs[0]->xxxxx->granulenumber = geo_input->xxxxx->granulenumber;
	outputs[1]->xxxxx->granulenumber = geo_input->xxxxx->granulenumber;
	
	outputs[0]->xxxxx->instrumentscansprocessed = (int32)n_scans_corrected;
	outputs[1]->xxxxx->instrumentscansprocessed = (int32)n_scans_corrected;

        if(PGS_MET_Init(MCF_SREF, METAHANDLES)!=PGS_S_SUCCESS) {
          modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_Init", location);
          return (-1);
           }
        set_ECS_metadata(REFPROD, outputs[0]->xxxxx);
        PGS_MET_Write(COREHANDLES, "CoreMetadata.0", outputs[0]->sdid);
        PGS_MET_Write(ARCHIVEHANDLES, "ArchiveMetadata.0", outputs[0]->sdid);
        PGS_MET_Remove();
	
        if(PGS_MET_Init(MCF_SREFC, METAHANDLES)!=PGS_S_SUCCESS) {
          modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_Init", location);
          return (-1);
           }
        set_ECS_metadata(COARSEPROD, outputs[1]->xxxxx);
        PGS_MET_Write(COREHANDLES, "CoreMetadata.0", outputs[1]->sdid);
        PGS_MET_Write(ARCHIVEHANDLES, "ArchiveMetadata.0", outputs[1]->sdid);
        PGS_MET_Remove();
return(0);
 }


int night_process_metadata(int bands[], qavals_t *qavalues, dims_struct *outputs, dims_struct *rad1km_input, dims_struct *geo_input, input_metadata *metadata, int n_scans_corrected)
{
char location[50]="meta.c, night_process_metadata\0";
int i, j, ich, nline_1km; 
void put_qa_into_metadata(int bands[], qavals_t *qavalues, input_metadata *metadata, int lines);


        nline_1km = geo_input->nblines;

        /* Set all qa-related parts of the metadata */
        put_qa_into_metadata(bands, qavalues, metadata, nline_1km);

        /* And copy them from the generic metadata to the reflectance outputs' metadata */
        for(i=0;i<BANDPROCESS;i++) 
          for(j=0;j<16;j++) {
             outputs->xxxxx->qualityclasspercentage[i][j] = metadata->qualityclasspercentage[i][j];
	     }
        for(j=0;j<4;j++) {
          outputs->xxxxx->aerosol_clmh[j] =  metadata->aerosol_clmh[j];
	     }

        outputs->xxxxx->qapercentgoodquality = metadata->qapercentgoodquality;
        outputs->xxxxx->qapercentother = metadata->qapercentother;
        outputs->xxxxx->qapercentnotproducedcloud = metadata->qapercentnotproducedcloud;
        outputs->xxxxx->qapercentnotproducedother = metadata->qapercentnotproducedother;
        outputs->xxxxx->percentnotsea = metadata->percentnotsea;
        outputs->xxxxx->qapercentoutofbounds = metadata->qapercentoutofbounds;
        outputs->xxxxx->qapercentmissing = metadata->qapercentmissing;
        outputs->xxxxx->highsolarzenith = metadata->highsolarzenith;

        strcpy(outputs->xxxxx->seaprocessed, metadata->seaprocessed);
        strcpy(outputs->xxxxx->atmosphericcorrectionattempted, metadata->atmosphericcorrectionattempted);
        strcpy(outputs->xxxxx->adjacencycorrectionattempted, metadata->adjacencycorrectionattempted);
        strcpy(outputs->xxxxx->autoquality, metadata->autoquality);
 
        /* This copies INPUTPOINTERS. */
        cpy_inputpointer(outputs->xxxxx, metadata);

        /* Copy over all the RELEVANT metadata (GRINGs, etc).  *NOT* stuff like INPUTPOINTERS. */
        ECSmetadatacopy(outputs->xxxxx, rad1km_input->xxxxx);
		
        /* Make some localgranuleids. */
        strcpy(outputs->xxxxx->localgranuleidcoarse, create_localgranuleid(outputs->xxxxx->rbdt, VersionID, CRF_ESDTname)); 

        /* Copy the sci_state, sci_abnorm, granulenumber. */
	outputs->xxxxx->sci_abnorm    = geo_input->xxxxx->sci_abnorm;
	outputs->xxxxx->sci_state     = geo_input->xxxxx->sci_state;
	outputs->xxxxx->granulenumber = geo_input->xxxxx->granulenumber;
	
	outputs->xxxxx->instrumentscansprocessed = (int32)n_scans_corrected;

        if(PGS_MET_Init(MCF_SREFC, METAHANDLES)!=PGS_S_SUCCESS) {
          modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_Init", location);
          return (-1);
           }
        set_ECS_metadata(COARSEPROD, outputs->xxxxx);
        PGS_MET_Write(COREHANDLES, "CoreMetadata.0", outputs->sdid);
        PGS_MET_Write(ARCHIVEHANDLES, "ArchiveMetadata.0", outputs->sdid);
        PGS_MET_Remove();

return(0);
 }


PGSt_SMF_status readECSmetadata_string(PGSt_PC_Logical file, PGSt_integer version,
			    char *hdfattr, char *field, void *out)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************
*/
{
  
  /* read metadata field from input file */  
  if(PGS_MET_GetPCAttr(file, version, hdfattr, field, &out) != PGS_S_SUCCESS){ 
    sprintf(msg, "PGS_MET_GetPCAttr retrieving field %s from string %s in "
	    "file LUN %ld and version %d", field, hdfattr, (LINT)file, (int32)version);
    modsmf((PGSt_SMF_code)MODIS_W_SDP_METADATA, msg, "meta.c, readECSmetadata_string\0");
    return(MODIS_W_SDP_METADATA);
  }
  
  return(MODIS_S_SUCCESS);
}


PGSt_SMF_status readECSmetadata_numerical(PGSt_PC_Logical file, PGSt_integer version,
			    char *hdfattr, char *field, void *out)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************
*/
{
  
  /* read metadata field from input file */  
  if(PGS_MET_GetPCAttr(file, version, hdfattr, field, out) != PGS_S_SUCCESS){ 
    sprintf(msg, "PGS_MET_GetPCAttr retrieving field %s from string %s in "
	    "file LUN %ld and version %d", field, hdfattr, (LINT)file, (int32)version);
    modsmf((PGSt_SMF_code)MODIS_W_SDP_METADATA, msg, "meta.c, readECSmetadata_numerical\0");
    return(MODIS_W_SDP_METADATA);
  }
  
  return(MODIS_S_SUCCESS);
}


int get_ESDT_names(int sattype, char *REF_ESDTname, char *CRF_ESDTname, char *VersionID)
/*
!C********************************************************************
!Description: get_ESDT_names() 

     Reads ESDT metadatum "ShortName" from the MCF files; also get metadatum "VersionID"

!Input Parameters:
     sattype           satellite type the program is expecting

!Output Parameters:   
     REF_ESDTname      the ESDT ShortName from the MOD09 MCF (as listed by
                       LUN MCF_SREF in the pcf).
     CRF_ESDTname      the ESDT ShortName from the MOD09CRS MCF (as listed by
                       LUN MCF_SREFC in the pcf).
     metadata          metadata structure into which "VersionID" is read.  Note this 
                       feature is currently not implemented, but will be implemented
                       when ESDT versionID is to be read from MCF (May 1, 2001).

!Input/Output Parameters:


!Returns:
      0   success
      1   failed reading the MOD09 MCF file
     -1   failed initializing the MOD09 MCF file
      2   failed reading the MOD09CRS MCF file
     -2   failed initializing the MOD09CRS MCF file
      4   various inconsistencies among ESDT names and/or sattype

!Revision History:
   Revision 1.0  
   Jim Ray/SSAI/NASA's GSFC Code 923
   Initial revision.
 
!Team-unique Header: 

  This software is created by the MODIS Land Science Team and Godard
  Space Flight Center Code 923 under contract NAS5-31752/002.

!References and Credits:
  Written by Jim Ray
  Science Systems and Applications Inc.
  NASA's GSFC Code 923
  Greenbelt, MD 20771
  jim@cratmos.gsfc.nasa.gov
  phone: (301) 614-6684


!Design Notes:

!END  
*********************************************************************/
{
char location[50]="meta.c, get_ESDT_names\0";
char *word;
int j, vers;

word = (char *)malloc(100);

if(PGS_MET_Init(MCF_SREF, METAHANDLES)!=PGS_S_SUCCESS) {
     modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_Init", location);
     return (-1);
     }
if(PGS_MET_GetSetAttr(COREHANDLES, Mfield_name[SHORTNAME], &word)!=PGS_S_SUCCESS) {
     modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_GetSetAttr", location);
     return (1);
     }
strcpy(REF_ESDTname, word);
strcat(word, "\0");

/* New, 27-MAR-01 */
  if(PGS_MET_GetSetAttr(COREHANDLES, Mfield_name[VERSIONID], &vers)!=PGS_S_SUCCESS) {
     modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_GetSetAttr", location);
     return (1);
     }
  PGS_MET_Remove();

  if(vers<0 || vers>999){
     modsmf(MODIS_E_BAD_INPUT_PARAMETER, "LOCALGRANULEID versionID "
	    "number is out of range (1-999)", location);
  }
  sprintf(VersionID, "%03d\0", vers);


/* end new stuff, 27-MAR-01 */

if(PGS_MET_Init(MCF_SREFC, METAHANDLES)!=PGS_S_SUCCESS) {
     modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_Init", location);
     return (-2);
     }
if(PGS_MET_GetSetAttr(COREHANDLES, Mfield_name[SHORTNAME], &word)!=PGS_S_SUCCESS) {
     modsmf((PGSt_SMF_code)MODIS_E_SDP_ERROR, "PGS_MET_GetSetAttr", location);
     return (2);
     }
strcpy(CRF_ESDTname, word);
strcat(word, "\0");
PGS_MET_Remove();


/* Make sure all ESDTnames are internally consistent */
if ((REF_ESDTname[0] != CRF_ESDTname[0]) || ((REF_ESDTname[1] != CRF_ESDTname[1]) ))
{
     modsmf((PGSt_SMF_code)MODIS_E_BAD_MCF, "Inconsistent information", location);
     return (4);
}

if ((REF_ESDTname[0] != 'M') || 
   ((REF_ESDTname[1] != 'O') && (REF_ESDTname[1] != 'Y'))) {
     modsmf((PGSt_SMF_code)MODIS_E_BAD_MCF, "Unrecognizable ESDT ShortName", location);
     return (4);
}

if (((REF_ESDTname[1] == 'O') && (sattype != 0)) ||
    ((REF_ESDTname[1] == 'Y') && (sattype != 1)))
{
     modsmf((PGSt_SMF_code)MODIS_E_PCF_MCF_MISMATCH, " ", location);
     return (4);
}

return(0);
}


char *create_localgranuleid(char *timecodea, char *version, char *esdt)
/*
!C*****************************************************************************
!Description: creates LOCALGRANULEID metadata filed string

!Input Parameters:
timecodea -- CCSDS ascii time code A string indicating the start time of the 
             output granule
version -- version number of the output granule 
esdt -- process esdt name

!Output Parameters: none

!Input/Output Parameters: none

!Returns: LOCALGRANULEID on success
          LOCALGRANULEIDDEFAULT failure

!Revision History:
	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

	Modified to include an approximate processing date and time
	in return string, per latest ECS metadata requirement (for MEBS);
	by R. Cember, SDST, 14 Nov 97.

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes:
1. LOCALGRANULEID created per the V2 MODIS SCF software delivery guide.


!END***************************************************************************
*/

/* Mod by R. Cember, SDST, 14 Nov 97 */

{

  char timecodeb[TIMECODEBSIZE];
  char year[PGSd_PC_VALUE_LENGTH_MAX], day[PGSd_PC_VALUE_LENGTH_MAX];
  char hour[PGSd_PC_VALUE_LENGTH_MAX], minute[PGSd_PC_VALUE_LENGTH_MAX];
  static char id[PGSd_PC_VALUE_LENGTH_MAX];

/* Mod by R. Cember, SDST, 14 Nov 97 */
  time_t time_sec;
  struct tm *time_ptr = NULL;

  char proc_year[PGSd_PC_VALUE_LENGTH_MAX], proc_day[PGSd_PC_VALUE_LENGTH_MAX];
  char proc_hour[PGSd_PC_VALUE_LENGTH_MAX], proc_minute[PGSd_PC_VALUE_LENGTH_MAX];  
  char proc_second[PGSd_PC_VALUE_LENGTH_MAX];
/* End mod */

  if(PGS_TD_ASCIItime_AtoB(timecodea,timecodeb)!=PGS_S_SUCCESS){
    modsmf(MODIS_F_SDP_ERROR, "PGS_TD_ASCIItime_AtoB", "meta.c, create_localgranuleid\0");

    return(LOCALGRANULEIDDEFAULT);
  }

  strncpy(year, timecodeb, 4);
  year[4]='\0';
  strncpy(day, &(timecodeb[5]), 3);
  day[3]='\0';
  strncpy(hour, &(timecodeb[9]), 2);
  hour[2]='\0';
  strncpy(minute, &(timecodeb[12]), 2);
  minute[2]='\0';

/* Mod by R. Cember, SDST, 14 Nov 97 */
  time(&time_sec);
  time_ptr = gmtime(&time_sec);

  sprintf( proc_year, "%.4d", 1900+(time_ptr->tm_year) );
  sprintf( proc_day, "%.3d", (time_ptr->tm_yday)+1 );
  sprintf( proc_hour, "%.2d", time_ptr->tm_hour );
  sprintf( proc_minute, "%.2d", time_ptr->tm_min );
  sprintf( proc_second, "%.2d", time_ptr->tm_sec );

  sprintf(id, "%s.A%s%s.%s%s.%s.%s%s%s%s%s.hdf", esdt, year, day, hour, minute, version, 
      proc_year, proc_day, proc_hour, proc_minute, proc_second );

  return(id);

}


int getECSsattype(PGSt_PC_Logical fileid, PGSt_integer version)
/*
!C********************************************************************
!Description: getECSsattype() 

     Reads LOCALGRANULEID from an input file, determines satellite type

!Input Parameters:


!Output Parameters:


!Input/Output Parameters:


!Returns:
      0   for Terra data
      1   for Aqua data
     -1   for neither; this routine DOES NOT assume input is Terra if it cannot tell.
     -2   other: in all likelihood, a coding error

!Revision History:
   Revision 1.0  
   Jim Ray/SSAI/NASA's GSFC Code 923
   Initial revision.
 
!Team-unique Header: 

  This software is created by the MODIS Land Science Team and Godard
  Space Flight Center Code 923 under contract NAS5-31752/002.

!References and Credits:
  Written by Jim Ray
  Science Systems and Applications Inc.
  NASA's GSFC Code 923
  Greenbelt, MD 20771
  jim@cratmos.gsfc.nasa.gov
  phone: (301) 614-6684


!Design Notes:

!END  
*********************************************************************/
{
char location[50]="meta.c, getECSsattype\0";
PGSt_integer verr;
char msg[PGS_SMF_MAX_MSG_SIZE];
char name[PGS_SMF_MAX_MSG_SIZE] = {'\0'};

  verr=version;
  if(readECSmetadata_string(fileid, verr, CORES, "LOCALGRANULEID", name) != MODIS_S_SUCCESS){
    sprintf(msg, "'LOCALGRANULEID' metadata field was not found within the file with lun %ld and version %d", 
	    (LINT)fileid, (int32)version);
    modsmf(MODIS_W_GET_METADATA, msg, location);
    return(-1);
  } 

  if ((name[0] != 'M') || ((name[1] != 'O') && (name[1] != 'Y'))) {
    sprintf(msg, "LOCALGRANULEID metadatum, '%s', not recognized; file lun %ld, version %d", 
                  name, (LINT)fileid, (int32)version);
    modsmf(MODIS_W_GET_METADATA, msg, location);
    return(-1);
  }
  
  if (name[1] == 'O')      return(0);
  if (name[1] == 'Y')      return(1);
  return(-2);
}

PGSt_SMF_status get_inputpointers(input_metadata *metadata, dims_struct **inputs)
{
int i;
char ur[PGSd_PC_UREF_LENGTH_MAX];
char sfilename[PGSd_PC_FILE_PATH_MAX];
char filename[PGSd_PC_FILE_PATH_MAX];

  for(i=0;i<ALL_L1B_FILES;i++) {
    if (inputs[i]->OK == 0) {
       strcpy(sfilename, inputs[i]->filename);
       
       if(rmpath(sfilename)!=MODIS_S_SUCCESS){
           modsmf((PGSt_SMF_code)MODIS_E_FUNCTION_ERROR, "Failure from rmpath()","meta.c, get_inputpointers\0");
           return((PGSt_SMF_status)MODIS_E_FUNCTION_ERROR);
           }
       grow_Mstringarray(metadata->inputpointer, PGSd_PC_FILE_NAME_MAX, sfilename);
       grow_Mstringarray(metadata->systemfilename, PGSd_PC_UREF_LENGTH_MAX, sfilename);
       if (i==GEO_FILE) {
           grow_Mstringarray(metadata->ancinputpointer, PGSd_PC_FILE_NAME_MAX, sfilename);
           grow_Mstringarray(metadata->ancsystemfilename, PGSd_PC_FILE_NAME_MAX, sfilename);
	   }
          }
       }
return(0);
}


void pop_inputpointers(input_metadata *metadata, dims_struct **inputs)
{
int i;

  for(i=0;i<ALL_L1B_FILES;i++) {
    if (inputs[i]->OK == 0) {
       shrink_Mstringarray(metadata->inputpointer);
       shrink_Mstringarray(metadata->systemfilename);
          }
       }
  for(i=0;i<4;i++) {  /* Pop the next 4 also, which are ancillary data filenames */
       shrink_Mstringarray(metadata->inputpointer);
       shrink_Mstringarray(metadata->systemfilename);
       }
  shrink_Mstringarray(metadata->ancinputpointer);
  shrink_Mstringarray(metadata->ancsystemfilename);
}

int32 format_to_int32(char *format)
/*
!C*****************************************************************************
!Description:
        Turns standard "major.minor.update" format string into 32-bit integer, so
	we can compare version numbers simply.

!Input Parameters:
        format       the "major.minor.update" format string
	
!Output Parameters: None
        
!Returns:
        format converted to integer (major*256*256 + minor*256 + update),
	or -1 in case of failure.   
           

!Revision History:
        Original version Februray, 2001, J. Ray, SSAI (jim@cratmos.gsfc.nasa.gov).
	
!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062


!References and Credits

!Design Notes    

!END***************************************************************************
*/
{
int i, k, n_periods;
int major,minor,update;
char *text = NULL;
int32 answer;

k = strlen(format);

/* Count the number of periods; there should be two. */
for (i=0,n_periods=0;i<k;i++) 
   if (format[i] == '.') n_periods++;
if (n_periods != 2) return ((int32)-1);

/* Parse the format string into major, minor, update. */
text = strtok(format, ".");
if (strlen(text) > 0) major = atoi(text);

text = strtok(NULL, ".");
if (strlen(text) > 0) minor = atoi(text);

text = strtok(NULL, ".");
if (strlen(text) > 0) update = atoi(text);

/* compute integer value */
answer = (int32)((major*256*256) + (minor*256) + update);
return(answer);

}


PGSt_SMF_status getECSfileinfo(PGSt_PC_Logical file_lun, PGSt_integer ver, input_metadata *metadata)
/*
!C******************************************************************

!Description: retrieves filename and UR.   stores them within inputpointer
              and systemfilename structures;


!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END**************************************************************************** */
{

  char ur[PGSd_PC_UREF_LENGTH_MAX];
  char sfilename[PGSd_PC_FILE_PATH_MAX];
  char filename[PGSd_PC_FILE_PATH_MAX];
  PGSt_integer tver;
  char location[100]="meta.c, getECSfileinfo\0";

  tver=ver;

  if(PGS_PC_GetReference(file_lun, &tver, filename)!=PGS_S_SUCCESS){
    sprintf(msg, "LUN %d, version %d", (int32)file_lun, ver);
    modsmf((PGSt_SMF_code)MODIS_E_GETREF_FAIL, msg, location);	   
    return(MODIS_E_GETREF_FAIL);
  }

  strcpy(sfilename, filename);

  if(rmpath(sfilename)!=MODIS_S_SUCCESS){
    modsmf((PGSt_SMF_code)MODIS_E_FUNCTION_ERROR, "Failure from rmpath()", location);
    return((PGSt_SMF_status)MODIS_E_FUNCTION_ERROR);
  }

  tver=ver;
  if(PGS_PC_GetUniversalRef(file_lun, &tver, ur)!=PGS_S_SUCCESS){
    sprintf(msg, "LUN %d, version %d", (int32)file_lun, ver);
    modsmf((PGSt_SMF_code)MODIS_E_GETUR_FAIL, msg, location);
    return(MODIS_E_GETUR_FAIL);
  }

  grow_Mstringarray(metadata->inputpointer, PGSd_PC_FILE_NAME_MAX, ur);
  grow_Mstringarray(metadata->systemfilename, PGSd_PC_UREF_LENGTH_MAX, sfilename);
     
  return(MODIS_S_SUCCESS);

}



PGSt_SMF_status getECSmetadata(PGSt_PC_Logical fileid, PGSt_integer version, input_metadata *input)
/*
!C********************************************************************
!Description: getECSmetadata() 

     Reads and sets all necessary ECS metadata from an input file.

!Input Parameters:


!Output Parameters:


!Input/Output Parameters:


!Returns:

!Revision History:
   Revision 1.0  
   Paul Fisher/SSAI/NASA's GSFC Code 923
   Initial revision.
 
!Team-unique Header: 

  This software is created by the MODIS Land Science Team and Godard
  Space Flight Center Code 923 under contract NAS5-31752/002.

!References and Credits:
  Written by Paul S. Fisher
  Science Systems and Applications Inc.
  NASA's GSFC Code 923
  Greenbelt, MD 20771
  fisher@ltpmail.gsfc.nasa.gov
  phone: (301) 286-1980


!Design Notes:

!END  
*********************************************************************/
{
  PGSt_SMF_status status;
  PGSt_integer ival, gringival[GRINGPOINTS];
  PGSt_double dval, gringdval[GRINGPOINTS];
  int32 gp;
  int32 storeversion;
  char time[PGSd_PC_VALUE_LENGTH_MAX], date[PGSd_PC_VALUE_LENGTH_MAX];
  char string_string[PGSd_PC_VALUE_LENGTH_MAX];
  /* 10-MAR-99, Jim Ray */
  int i;
  char AUTOQUALFLAG[PGSd_PC_VALUE_LENGTH_MAX] = {'\0'};
  char versionID[PGSd_PC_VALUE_LENGTH_MAX] = {'\0'};
  char *formatstring;
  int32 format_to_int32(char *format);
  char location[50]="meta.c, getECSmetadata\0";

  /* storage is zero based, while version numbers are 1 based */
  storeversion=version-1;
  formatstring = (char *)malloc(15);

  /* 10-MAR-99, Jim Ray */
  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[AUTOMATICQUALITYFLAG], AUTOQUALFLAG)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "Warning: %s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[AUTOMATICQUALITYFLAG], (LINT)fileid, (int32)version);
    modsmf(MODIS_W_GET_METADATA, msg, location);
  } 
  if (strlen(AUTOQUALFLAG)) {  /* if there is a flag there... */
     for (i=0;i<strlen(AUTOQUALFLAG);i++) AUTOQUALFLAG[i] = toupper(AUTOQUALFLAG[i]);
     if (strcmp(AUTOQUALFLAG,"PASSED")) {
       sprintf(msg, "Warning: %s metadata field value = '%s', file lun %ld, version %d",
               Mfield_name[AUTOMATICQUALITYFLAG], AUTOQUALFLAG, (LINT)fileid, (int32)version);
       modsmf(MODIS_W_GET_METADATA, msg, location);
     }
  }
  /* For now will just give warnings ... */
  /* End of 10-MAR-99 material */
  
  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[RANGEBEGINNINGTIME], time)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[RANGEBEGINNINGTIME], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  
  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[RANGEBEGINNINGDATE], date)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[RANGEBEGINNINGDATE], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  
  /* create datetime string (CCSDS TIME CODE A) */
  sprintf(input->rbdt, "%sT%sZ", date, time);
  
  /* store rbdt as tai */
  if(PGS_TD_UTCtoTAI(input->rbdt, &(input->rbdt_tai))!=PGS_S_SUCCESS){
    sprintf(msg, "Unable to convert %s to tai time", input->rbdt);
    modsmf(MODIS_E_BAD_TIME_CONV, msg, location);
    return(MODIS_E_BAD_TIME_CONV);
  }
  
  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[RANGEENDINGTIME], time)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[RANGEENDINGTIME], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  
  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[RANGEENDINGDATE], date)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[RANGEENDINGDATE], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  
  /* create datetime string (CCSDS TIME CODE A) */
  sprintf(input->redt, "%sT%sZ", date, time);
    
  if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[ORBITNUMBER], &ival)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[ORBITNUMBER], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  input->orbitnumber=(int32)ival;
  
    
  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[DAYNIGHTFLAG], string_string)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[DAYNIGHTFLAG], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  strcpy(input->daynightflag, string_string);  

  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[LOCALGRANULEID], string_string)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[LOCALGRANULEID], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  strcpy(input->localgranuleid, string_string);  
      
  /* NOTE that all input files will not contain the bounding coordinates and not
     all files that do will contain them in the same place */
  if(fileid==CLD){
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[NORTHBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS) input->northboundingcoordinate=dval;
    
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[SOUTHBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS)input->southboundingcoordinate=dval;
    
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[EASTBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS)input->eastboundingcoordinate=dval;
    
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[WESTBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS)input->westboundingcoordinate=dval;
  }
  
  else /*if(fileid!=RAD1 && fileid!=RAD500 && fileid!=RAD250)*/{
    if(readECSmetadata_numerical(fileid, version, ARCHIVES, Mfield_name[NORTHBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS) input->northboundingcoordinate=dval;
    
    if(readECSmetadata_numerical(fileid, version, ARCHIVES, Mfield_name[SOUTHBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS)input->southboundingcoordinate=dval;
    
    if(readECSmetadata_numerical(fileid, version, ARCHIVES, Mfield_name[EASTBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS)input->eastboundingcoordinate=dval;
    
    if(readECSmetadata_numerical(fileid, version, ARCHIVES, Mfield_name[WESTBOUNDINGCOORDINATE], &dval)==
       MODIS_S_SUCCESS)input->westboundingcoordinate=dval;
  }

  if(fileid!=CLD){
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[GRINGPOINTLATITUDE], gringdval)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[GRINGPOINTLATITUDE], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
    }
    for(gp=0;gp<GRINGPOINTS;gp++){
      input->gringpointlatitude[gp]=(float64)gringdval[gp];
    } /* for(gp... */
    
    
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[GRINGPOINTLONGITUDE], gringdval)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[GRINGPOINTLONGITUDE], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
    }
    for(gp=0;gp<GRINGPOINTS;gp++){
      input->gringpointlongitude[gp]=(float64)gringdval[gp];
    } /* for(gp... */
    
    
    if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[GRINGPOINTSEQUENCENO], gringival)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[GRINGPOINTSEQUENCENO], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
    }
    for(gp=0;gp<GRINGPOINTS;gp++){
      input->gringpointsequenceno[gp]=(int32)gringival[gp];
    } /* for(gp... */
    
    
    
    if(readECSmetadata_string(fileid, version, CORES, Mfield_name[EXCLUSIONGRINGFLAG], string_string)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[EXCLUSIONGRINGFLAG], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
    }
    strcpy(input->exclusiongringflag, string_string);
  }
  
  
  /* only take granule number off of the geolocation input file */
  
  if(fileid==GEO){
    if(readECSmetadata_string(fileid, version, CORES, Mfield_name[INPUTGRANULENUMBER], string_string)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[INPUTGRANULENUMBER], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
    }
    input->granulenumber=atoi(string_string);

    if(readECSmetadata_string(fileid, version, CORES, Mfield_name[SCI_STATE], string_string)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[SCI_STATE], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
    }
    input->sci_state=atoi(string_string);


   if(readECSmetadata_string(fileid, version, CORES, Mfield_name[SCI_ABNORM], string_string)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[SCI_ABNORM], (LINT)fileid, (int32)version);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return(MODIS_E_GET_METADATA);
   }
   input->sci_abnorm=atoi(string_string);
  }


  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[EQUATORCROSSINGDATE], string_string)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[EQUATORCROSSINGDATE], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  strcpy(input->equatorcrossingdate, string_string);  

  if(readECSmetadata_string(fileid, version, CORES, Mfield_name[EQUATORCROSSINGTIME], string_string)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[EQUATORCROSSINGTIME], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  strcpy(input->equatorcrossingtime, string_string);  

  if(readECSmetadata_numerical(fileid, version, CORES, Mfield_name[EQUATORCROSSINGLONGITUDE], &dval)!=
     MODIS_S_SUCCESS){
    sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	    Mfield_name[EQUATORCROSSINGLONGITUDE], (LINT)fileid, (int32)version);
    modsmf(MODIS_E_GET_METADATA, msg, location);
    return(MODIS_E_GET_METADATA);
  }
  input->equatorcrossinglongitude=(float64)dval;

  /* Right here, get real version information from the L1B (code from get_L1B_format()) */
  if(readECSmetadata_string(fileid, version, CORES, "PGEVERSION", versionID)!=MODIS_S_SUCCESS){
    sprintf(msg, "PGEVERSION metadata field was not found, file lun %ld, version %d; will assume new version", 
           (LINT)fileid, (int32)version);
    modsmf(MODIS_W_GET_METADATA, msg, location);
    strcpy(formatstring, "2.5.4\0");
    input->int_pgeversion = format_to_int32(formatstring);  /* assume the L1B format currently in ops (09-FEB-01) */
    }
  
  if (!strlen(versionID)) {  /* versionID was blank (?) */
    sprintf(msg, "PGEVERSION metadata field is blank, lun %ld, version %d", (LINT)fileid, (int32)version);
    modsmf(MODIS_W_GET_METADATA, msg, location);
    strcpy(formatstring, "2.5.4\0");
    input->int_pgeversion = format_to_int32(formatstring);  /* assume the L1B format currently in ops (09-FEB-01) */
        }
        
  if (!strcmp(versionID,"2")) {  /* only "2", will assume old version */
    strcpy(formatstring, "2.0.0\0");
    input->int_pgeversion  = format_to_int32(formatstring);
	}
  else 
    input->int_pgeversion  = format_to_int32(versionID);
   
  free(formatstring);

  
  return(MODIS_S_SUCCESS);
}

void setattr_wrapper_string(PGSt_MET_handle handle, char *namestring,
			    void *value)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************
*/
{ 
  if(PGS_MET_SetAttr(handle, namestring, &value)!=PGS_S_SUCCESS){
    sprintf(msg, "PGS_MET_SetAttr setting %s", namestring);
    modsmf((PGSt_SMF_code)MODIS_E_SDP_METADATA, msg,  "meta.c, setattr_wrapper_string\0");
  }
  
  return;
}


void setattr_wrapper_numerical(PGSt_MET_handle handle, char *namestring,
			    void *value)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov

!Design Notes


!END****************************************************************************
*/
{
  if(PGS_MET_SetAttr(handle, namestring, value)!=PGS_S_SUCCESS){
    sprintf(msg, "PGS_MET_SetAttr setting %s", namestring);
    modsmf((PGSt_SMF_code)MODIS_E_SDP_METADATA, msg,  "meta.c, setattr_wrapper_numerical\0");
  }
  
  return; 
}


void setattr_wrapper_array(PGSt_MET_handle handle, char *namestring,
		     void **value)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov

!Design Notes


!END****************************************************************************
*/
{
  if(PGS_MET_SetAttr(handle, namestring, value)!=PGS_S_SUCCESS){
    sprintf(msg, "PGS_MET_SetAttr setting %s", namestring);
    modsmf((PGSt_SMF_code)MODIS_E_SDP_METADATA, msg,  "meta.c, setattr_wrapper_array\0");
  }

  return;
}


void set_ECS_metadata(filetype_t filetype, input_metadata *metadata)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************
*/
{
  /* string to hold numerical values */
  char vstring[PGSd_PC_VALUE_LENGTH_MAX];

  char rbdtdate[PGSd_PC_VALUE_LENGTH_MAX], redtdate[PGSd_PC_VALUE_LENGTH_MAX];
  char rbdttime[PGSd_PC_VALUE_LENGTH_MAX], redttime[PGSd_PC_VALUE_LENGTH_MAX];
  
  /* New, 24-FEB-00: a routine to check metadata for fill values, inform user of fill values
     (metadata was never set), and replace fill values with acceptable values. */
  void clear_fill(input_metadata *metadata);
  
  clear_fill(metadata);

  /* things that get set the same for all file types */

  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[NORTHBOUNDINGCOORDINATE], &(metadata->northboundingcoordinate));
  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[SOUTHBOUNDINGCOORDINATE], &(metadata->southboundingcoordinate));
  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[EASTBOUNDINGCOORDINATE], &(metadata->eastboundingcoordinate));
  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[WESTBOUNDINGCOORDINATE], &(metadata->westboundingcoordinate));

  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[SCI_STATE_OUT], &(metadata->sci_state));
  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[SCI_ABNORM_OUT], &(metadata->sci_abnorm));

  setattr_wrapper_numerical(COREHANDLES, Mfield_name[EQUATORCROSSINGLONGITUDE], &(metadata->equatorcrossinglongitude));
  setattr_wrapper_string(COREHANDLES, Mfield_name[EQUATORCROSSINGTIME], metadata->equatorcrossingtime);
  setattr_wrapper_string(COREHANDLES, Mfield_name[EQUATORCROSSINGDATE], metadata->equatorcrossingdate);
  /* As of latest MCF (30-SEP-99), internal cloud mask data is not to be written to metadata.
  setattr_wrapper_string(ARCHIVEHANDLES, Mfield_name[USEDINTCLOUDMASK], metadata->used_int_cloud_mask); */

  split_datetime(metadata->rbdt, rbdtdate, rbdttime);
  setattr_wrapper_string(COREHANDLES, Mfield_name[RANGEBEGINNINGDATE], rbdtdate);
  setattr_wrapper_string(COREHANDLES, Mfield_name[RANGEBEGINNINGTIME], rbdttime);
 
  split_datetime(metadata->redt, redtdate, redttime);
  setattr_wrapper_string(COREHANDLES, Mfield_name[RANGEENDINGDATE], redtdate);
  setattr_wrapper_string(COREHANDLES, Mfield_name[RANGEENDINGTIME], redttime);
  


  setattr_wrapper_numerical(COREHANDLES, Mfield_name[ORBITNUMBER], &(metadata->orbitnumber));
  setattr_wrapper_numerical(COREHANDLES, Mfield_name[GRINGPOINTLATITUDE], metadata->gringpointlatitude);
  setattr_wrapper_numerical(COREHANDLES, Mfield_name[GRINGPOINTLONGITUDE], metadata->gringpointlongitude);
  setattr_wrapper_numerical(COREHANDLES, Mfield_name[GRINGPOINTSEQUENCENO], metadata->gringpointsequenceno);
  setattr_wrapper_string(COREHANDLES, Mfield_name[EXCLUSIONGRINGFLAG], metadata->exclusiongringflag);

  setattr_wrapper_string(COREHANDLES, Mfield_name[DAYNIGHTFLAG], metadata->daynightflag);
  setattr_wrapper_string(COREHANDLES, Mfield_name[REPROCESSINGACTUAL], metadata->reprocessingactual);
  setattr_wrapper_string(COREHANDLES, Mfield_name[REPROCESSINGPLANNED], metadata->reprocessingplanned);
  setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[INSTRUMENTSCANSPROCESSED], &(metadata->instrumentscansprocessed));
  setattr_wrapper_string(COREHANDLES, Mfield_name[PGEVERSION], metadata->pgeversion);

  setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTGOODQUALITY], 
			 metadata->attributenameqapercentgoodquality);
  setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTOTHERQUALITY], 
			 metadata->attributenameqapercentotherquality);
  setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTNOTPRODUCEDCLOUD], 
			 metadata->attributenameqapercentnotproducedcloud);
  setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTNOTPRODUCEDOTHER], 
			 metadata->attributenameqapercentnotproducedother);


  setattr_wrapper_string(COREHANDLES, Mfield_name[ANCILLARYINPUTTYPE], ANCILLARYINPUTTYPEDEFAULT);

  /* As of 20-FEB-02: need to write ProcessingEnvironment to archivemetadata */
  setattr_wrapper_string(ARCHIVEHANDLES, Mfield_name[PROCESSINGENVIRONMENT], ProcessingEnvironment); 
    
/*************/
    setattr_wrapper_string(COREHANDLES, Mfield_name[AUTOMATICQUALITYFLAG], metadata->autoquality);

    /* As of latest MCF (30-SEP-99), parameter name is not to be written to MOD09/MOD09CRS metadata.
    if(filetype == REFPROD)    
        setattr_wrapper_string(COREHANDLES, Mfield_name[PARAMETERNAME], 
			       PARAMETERNAMESRDEFAULT); 
    else if(filetype == COARSEPROD)    
        setattr_wrapper_string(COREHANDLES, Mfield_name[PARAMETERNAME], 
			       PARAMETERNAMECSRDEFAULT); 
    else {
    }
    */

    setattr_wrapper_string(COREHANDLES, Mfield_name[AUTOMATICQUALITYFLAGEXPLANATION], 
			   AUTOMATICQUALITYFLAGEXPLANATIONREFDEFAULT);

    if(metadata->inputpointer->marray)
      setattr_wrapper_array(COREHANDLES, Mfield_name[INPUTPOINTER], 
			    (void **)metadata->inputpointer->marray);
    
    if(metadata->systemfilename->marray)
      setattr_wrapper_array(ARCHIVEHANDLES, Mfield_name[SYSTEMFILENAME], 
			    (void **)metadata->systemfilename->marray);
    if(metadata->ancinputpointer->marray)
      { 
	setattr_wrapper_array(COREHANDLES, Mfield_name[ANCILLARYINPUTPOINTER], 
			      (void **)metadata->ancinputpointer->marray);
      }      
    
    if(metadata->ancsystemfilename->marray)
      setattr_wrapper_array(ARCHIVEHANDLES, Mfield_name[ANCILLARYSYSTEMFILENAME], 
			    (void **)metadata->ancsystemfilename->marray);
        
    setattr_wrapper_string(ARCHIVEHANDLES, Mfield_name[ADJACENCYCORRECTIONATTEMPTED], metadata->adjacencycorrectionattempted);
    setattr_wrapper_string(ARCHIVEHANDLES, Mfield_name[ATMOSPHERICCORRECTIONATTEMPTED], metadata->atmosphericcorrectionattempted);
    setattr_wrapper_string(ARCHIVEHANDLES, Mfield_name[SEAPROCESSED], metadata->seaprocessed);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT250MBAND1], 
			   metadata->attributenameqapercentpooroutput250mband1);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT250MBAND2], 
			   metadata->attributenameqapercentpooroutput250mband2);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND1], 
			   metadata->attributenameqapercentpooroutput500mband1);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND2], 
			   metadata->attributenameqapercentpooroutput500mband2);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND3], 
			   metadata->attributenameqapercentpooroutput500mband3);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND4], 
			   metadata->attributenameqapercentpooroutput500mband4);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND5], 
			   metadata->attributenameqapercentpooroutput500mband5);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND6], 
			   metadata->attributenameqapercentpooroutput500mband6);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND7], 
			   metadata->attributenameqapercentpooroutput500mband7);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND1], 
			   metadata->attributenameqapercentpooroutput1kmband1);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND2], 
			   metadata->attributenameqapercentpooroutput1kmband2);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND3], 
			   metadata->attributenameqapercentpooroutput1kmband3);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND4], 
			   metadata->attributenameqapercentpooroutput1kmband4);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND5], 
			   metadata->attributenameqapercentpooroutput1kmband5);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND6], 
			   metadata->attributenameqapercentpooroutput1kmband6);
    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND7], 
			   metadata->attributenameqapercentpooroutput1kmband7);
    setattr_wrapper_string(ARCHIVEHANDLES, Mfield_name[BRDFCORRECTIONATTEMPTED], 
			   metadata->brdfcorrectionattempted);

    setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEGRANULENUMBER], 
			   metadata->attributenamegranulenumber);
    sprintf(vstring, "%d", metadata->granulenumber);
    setattr_wrapper_string(COREHANDLES, Mfield_name[GRANULENUMBER], 
			   vstring);
    
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE250MBAND1], 
			      metadata->qualityclasspercentage[BAND1_250M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE250MBAND2], 
			      metadata->qualityclasspercentage[BAND2_250M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND1], 
			      metadata->qualityclasspercentage[BAND1_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND2], 
			      metadata->qualityclasspercentage[BAND2_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND3], 
			      metadata->qualityclasspercentage[BAND3_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND4], 
			      metadata->qualityclasspercentage[BAND4_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND5], 
			      metadata->qualityclasspercentage[BAND5_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND6], 
			      metadata->qualityclasspercentage[BAND6_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE500MBAND7], 
			      metadata->qualityclasspercentage[BAND7_500M]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND1], 
			      metadata->qualityclasspercentage[BAND1_1KM]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND2], 
			      metadata->qualityclasspercentage[BAND2_1KM]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND3], 
			      metadata->qualityclasspercentage[BAND3_1KM]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND4], 
			      metadata->qualityclasspercentage[BAND4_1KM]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND5], 
			      metadata->qualityclasspercentage[BAND5_1KM]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND6], 
			      metadata->qualityclasspercentage[BAND6_1KM]);
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[QUALITYCLASSPERCENTAGE1KMBAND7], 
			      metadata->qualityclasspercentage[BAND7_1KM]);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND1_250M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT250MBAND1], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND2_250M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT250MBAND2], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND1_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND1], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND2_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND2], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND3_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND3], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND4_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND4], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND5_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND5], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND6_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND6], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND7_500M][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT500MBAND7], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND1_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND1], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND2_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND2], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND3_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND3], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND4_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND4], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND5_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND5], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND6_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND6], vstring);
    
    sprintf(vstring, "%d", CLEARFILL(metadata->qualityclasspercentage[BAND7_1KM][15]));
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTPOOROUTPUT1KMBAND7], vstring);
    
    sprintf(vstring, "%d", metadata->qapercentgoodquality);
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTGOODQUALITY], vstring);
    
    sprintf(vstring, "%d", metadata->qapercentother);
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTOTHER], vstring);
    
    sprintf(vstring, "%d", metadata->qapercentnotproducedcloud);
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTNOTPRODUCEDCLOUD], vstring);
    
    sprintf(vstring, "%d", metadata->qapercentnotproducedother);
    setattr_wrapper_string(COREHANDLES, Mfield_name[QAPERCENTNOTPRODUCEDOTHER], vstring);
    
    setattr_wrapper_numerical(COREHANDLES, Mfield_name[QAPERCENTOUTOFBOUNDS], 
			      &(metadata->qapercentoutofbounds));
    setattr_wrapper_numerical(COREHANDLES, Mfield_name[QAPERCENTMISSING], 
			      &(metadata->qapercentmissing));
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[PERCENTNOTSEA], 
			      &(metadata->percentnotsea));
			      
    if(filetype == REFPROD)    
       setattr_wrapper_string(COREHANDLES, Mfield_name[LOCALGRANULEID], 
			      metadata->localgranuleid);
    else if(filetype == COARSEPROD)    
       setattr_wrapper_string(COREHANDLES, Mfield_name[LOCALGRANULEID], 
			      metadata->localgranuleidcoarse);
    else if(filetype == IDSPROD)   /* for now */
       setattr_wrapper_string(COREHANDLES, Mfield_name[LOCALGRANULEID], 
			      metadata->localgranuleidcoarse);
    
     
			   
    setattr_wrapper_numerical(ARCHIVEHANDLES, Mfield_name[NUMBEROFHIGHSOLARZENITHPIXELS], 
                              &(metadata->highsolarzenith)); 
    setattr_wrapper_array(ARCHIVEHANDLES, Mfield_name[AEROSOLQUANTITY], 
			    (void **)metadata->aerosol_clmh);
			    
/*************/


  /* As of 13-MAR-02: will write processversion to LOCALVERSIONID; for now, keep
     additional attribute 22, "processversion", even though no longer necessary. */
  setattr_wrapper_string(COREHANDLES, Mfield_name[ATTRIBUTENAMEPROCESSVERSION], 
			 metadata->attributenameprocessversion);   
  setattr_wrapper_string(COREHANDLES, Mfield_name[PROCESSVERSION], 
			 metadata->processversion);   
  setattr_wrapper_string(COREHANDLES, Mfield_name[LOCALVERSIONID], 
			 metadata->processversion);   


  return;

}

void ECSmetadatacopy(input_metadata *to, input_metadata *from)
{
/* Meant for copying data from input files to output files; copies ONLY the relevant data */
int ig;
	strcpy(to->rbdt, from->rbdt);
	strcpy(to->redt, from->redt);
	to->orbitnumber=from->orbitnumber;
	to->granulenumber=from->granulenumber;
	strcpy(to->daynightflag,from->daynightflag);
	to->northboundingcoordinate=from->northboundingcoordinate;
	to->southboundingcoordinate=from->southboundingcoordinate;
	to->westboundingcoordinate=from->westboundingcoordinate;
	to->eastboundingcoordinate=from->eastboundingcoordinate;
	for(ig=0;ig<GRINGPOINTS;ig++){
	  to->gringpointlatitude[ig]=from->gringpointlatitude[ig];
	  to->gringpointlongitude[ig]=from->gringpointlongitude[ig];
	  to->gringpointsequenceno[ig]=from->gringpointsequenceno[ig];
	}
	strcpy(to->exclusiongringflag,from->exclusiongringflag);
	to->rbdt_tai=from->rbdt_tai;
	to->redt_tai=from->redt_tai;
	to->sci_state=from->sci_state;
	to->sci_abnorm=from->sci_abnorm;
	strcpy(to->equatorcrossingtime, from->equatorcrossingtime);
	strcpy(to->equatorcrossingdate, from->equatorcrossingdate);
	to->equatorcrossinglongitude=from->equatorcrossinglongitude;

}

void initECSmetadata(input_metadata *input)
{
int i, ig;
char location[50]="meta.c, initECSmetadata\0";

	strcpy(input->rbdt,RANGEBEGINNINGDATETIMEDEFAULT);
	strcpy(input->redt,RANGEENDINGDATETIMEDEFAULT);
	input->orbitnumber=ORBITNUMBERDEFAULT;
	input->granulenumber=GRANULENUMBERDEFAULT;
	strcpy(input->daynightflag,DAYNIGHTFLAGDEFAULT);
	input->northboundingcoordinate=COORDINATEDEFAULT;
	input->southboundingcoordinate=COORDINATEDEFAULT;
	input->westboundingcoordinate=COORDINATEDEFAULT;
	input->eastboundingcoordinate=COORDINATEDEFAULT;
	for(ig=0;ig<GRINGPOINTS;ig++){
	  input->gringpointlatitude[ig]=COORDINATEDEFAULT;
	  input->gringpointlongitude[ig]=COORDINATEDEFAULT;
	  input->gringpointsequenceno[ig]=GRINGPOINTSEQUENCENODEFAULT;
	}
	strcpy(input->exclusiongringflag,EXCLUSIONGRINGFLAGDEFAULT);
        input->instrumentscansprocessed=INSTRUMENTSCANSPROCESSEDDEFAULT;


  strcpy(input->seaprocessed,SEAPROCESSEDDEFAULT);
  strcpy(input->adjacencycorrectionattempted,ADJACENCYCORRECTIONATTEMPTEDDEFAULT);
  strcpy(input->atmosphericcorrectionattempted,ATMOSPHERICCORRECTIONATTEMPTEDDEFAULT);
  strcpy(input->attributenameqapercentgoodquality,ATTRIBUTENAMEQAPERCENTGOODQUALITYDEFAULT);
  strcpy(input->attributenameqapercentotherquality,ATTRIBUTENAMEQAPERCENTOTHERQUALITYDEFAULT);
  strcpy(input->attributenameqapercentnotproducedcloud,ATTRIBUTENAMEQAPERCENTNOTPRODUCEDCLOUDDEFAULT);
  strcpy(input->attributenameqapercentnotproducedother,ATTRIBUTENAMEQAPERCENTNOTPRODUCEDOTHERDEFAULT);
  strcpy(input->attributenameqapercentpooroutput250mband1,ATTRIBUTENAMEQAPERCENTPOOROUTPUT250MBAND1DEFAULT);
  strcpy(input->attributenameqapercentpooroutput250mband2,ATTRIBUTENAMEQAPERCENTPOOROUTPUT250MBAND2DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband1,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND1DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband2,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND2DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband3,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND3DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband4,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND4DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband5,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND5DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband6,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND6DEFAULT);
  strcpy(input->attributenameqapercentpooroutput500mband7,ATTRIBUTENAMEQAPERCENTPOOROUTPUT500MBAND7DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband1,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND1DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband2,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND2DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband3,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND3DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband4,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND4DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband5,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND5DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband6,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND6DEFAULT);
  strcpy(input->attributenameqapercentpooroutput1kmband7,ATTRIBUTENAMEQAPERCENTPOOROUTPUT1KMBAND7DEFAULT);
  strcpy(input->attributenamegranulenumber,ATTRIBUTENAMEGRANULENUMBERDEFAULT);
  strcpy(input->brdfcorrectionattempted,BRDFCORRECTIONATTEMPTEDDEFAULT);
strcpy(input->attributenameprocessversion, ADDITIONALATTRIBUTENAMEPROCESSVERSIONDEFAULT);

strcpy(input->autoquality, AUTOQUALITYDEFAULT);
  for(ig=0;ig<BANDPROCESS;ig++){
    for(i=0;i<16;i++)input->qualityclasspercentage[ig][i]=QUALITYCLASSPERCENTAGEDEFAULT;
  }

  input->qapercentgoodquality=MODLANDQADEFAULT;
  input->qapercentother=MODLANDQADEFAULT;
  input->qapercentnotproducedcloud=MODLANDQADEFAULT;
  input->qapercentnotproducedother=MODLANDQADEFAULT;
  input->percentnotsea=PERCENTNOTSEADEFAULT;
  input->percentland=PERCENTLANDDEFAULT;
  input->qapercentoutofbounds=ECSMANDATORYQADEFAULT;
  input->qapercentmissing=ECSMANDATORYQADEFAULT;

	strcpy(input->localgranuleid,LOCALGRANULEIDDEFAULT);
	strcpy(input->localgranuleidcoarse,LOCALGRANULEIDDEFAULT);
	input->rbdt_tai=TAIDEFAULT;
	input->redt_tai=TAIDEFAULT;
	input->sci_state=SCI_STATE_DEFAULT;
	input->sci_abnorm=SCI_ABNORM_DEFAULT;
	strcpy(input->equatorcrossingtime, EQUATORCROSSINGTIMEDEFAULT);
	strcpy(input->equatorcrossingdate, EQUATORCROSSINGDATEDEFAULT);
	input->equatorcrossinglongitude=EQUATORCROSSINGLONGITUDEDEFAULT;
    input->inputpointer=(marray_t *)AllocMem(sizeof(marray_t), location);
    input->systemfilename=(marray_t *)AllocMem(sizeof(marray_t), location);
    input->ancinputpointer=(marray_t *)AllocMem(sizeof(marray_t), location);
    input->ancsystemfilename=(marray_t *)AllocMem(sizeof(marray_t), location);
    
    input->inputpointer->marray=NULL;
    input->inputpointer->arcounter=0;
    input->systemfilename->marray=NULL;
    input->systemfilename->arcounter=0;
    input->ancinputpointer->marray=NULL;
    input->ancinputpointer->arcounter=0;
    input->ancsystemfilename->marray=NULL;
    input->ancsystemfilename->arcounter=0;
    for (ig=0;ig<4;ig++)  input->aerosol_clmh[ig] = 0;
    input->AutoQualityIsGood=AUTOGOOD;
    
    input->int_pgeversion = -1;

    /* obtain reprocessing metadata values from the pcf */
    if(PGS_PC_GetConfigData(PGEVERSIONNUMBERLUN, input->pgeversion)!=PGS_S_SUCCESS){
      sprintf(msg, "LUN: %d", PGEVERSIONNUMBERLUN);
      modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
      strcpy(input->pgeversion,PGEVERSIONDEFAULT);
    }
    
    strcpy(input->processversion,PROCESSVERSIONNUMBER);

    /* obtain reprocessing metadata values from the pcf */
    if(PGS_PC_GetConfigData(REPROCESSINGACTUALLUN, input->reprocessingactual)!=PGS_S_SUCCESS){
      sprintf(msg, "LUN: %d", REPROCESSINGACTUALLUN);
      modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
      strcpy(input->reprocessingactual,REPROCESSINGACTUALDEFAULT);
    }
    
    if(PGS_PC_GetConfigData(REPROCESSINGPLANNEDLUN,  input->reprocessingplanned)!=PGS_S_SUCCESS){
      sprintf(msg, "LUN: %d", REPROCESSINGPLANNEDLUN);
      modsmf((PGSt_SMF_code)MODIS_E_GETCONFIG_FAIL, msg, location);
      strcpy(input->reprocessingplanned,REPROCESSINGPLANNEDDEFAULT);
    }


}

void freeECSmetadata(input_metadata *metadata)
{
    while(metadata->inputpointer->arcounter>0)
      shrink_Mstringarray(metadata->inputpointer);
    while(metadata->ancinputpointer->arcounter>0)
      shrink_Mstringarray(metadata->ancinputpointer);
    while(metadata->systemfilename->arcounter>0)
      shrink_Mstringarray(metadata->systemfilename);
    while(metadata->ancsystemfilename->arcounter>0)
      shrink_Mstringarray(metadata->ancsystemfilename);
    if(metadata->systemfilename != NULL)free(metadata->systemfilename);
    if(metadata->inputpointer != NULL)free(metadata->inputpointer);
    if(metadata->ancinputpointer != NULL)free(metadata->ancinputpointer);
    if(metadata->ancsystemfilename != NULL)free(metadata->ancsystemfilename);
    
    free(metadata);

}

void cpy_inputpointer(input_metadata *to, input_metadata *from)
{
int i;
  
while(to->inputpointer->arcounter>0)  shrink_Mstringarray(to->inputpointer); 
if (from->inputpointer->arcounter>0){
    for (i=0;i<from->inputpointer->arcounter;i++)
       grow_Mstringarray(to->inputpointer, PGSd_PC_FILE_NAME_MAX, from->inputpointer->marray[i]);
   }
while(to->ancinputpointer->arcounter>0)  shrink_Mstringarray(to->ancinputpointer); 
if (from->ancinputpointer->arcounter>0){
    for (i=0;i<from->ancinputpointer->arcounter;i++)
       grow_Mstringarray(to->ancinputpointer, PGSd_PC_FILE_NAME_MAX, from->ancinputpointer->marray[i]);
   }
while(to->systemfilename->arcounter>0)  shrink_Mstringarray(to->systemfilename); 
if (from->systemfilename->arcounter>0){
    for (i=0;i<from->systemfilename->arcounter;i++)
       grow_Mstringarray(to->systemfilename, PGSd_PC_FILE_NAME_MAX, from->systemfilename->marray[i]);
   }
while(to->ancsystemfilename->arcounter>0)  shrink_Mstringarray(to->ancsystemfilename); 
if (from->ancsystemfilename->arcounter>0){
    for (i=0;i<from->ancsystemfilename->arcounter;i++)
       grow_Mstringarray(to->ancsystemfilename, PGSd_PC_FILE_NAME_MAX, from->ancsystemfilename->marray[i]);
   }
}

void ECScpy(input_metadata *to, input_metadata *from, int mode)
{
/* meant for copying internal copies of metadata, from one representation of metadata to another.  All metadata is relevant */
int ig;
void cpy_inputpointer(input_metadata *to, input_metadata *from);


if (mode) {/*  Copy one at the expense of the other */
  while(from->inputpointer->arcounter>0) {
    grow_Mstringarray(to->inputpointer, PGSd_PC_FILE_NAME_MAX, from->inputpointer->marray[from->inputpointer->arcounter-1]);
    shrink_Mstringarray(from->inputpointer); 
       }
  while(from->ancinputpointer->arcounter>0) {
    grow_Mstringarray(to->ancinputpointer, PGSd_PC_FILE_NAME_MAX, from->ancinputpointer->marray[from->ancinputpointer->arcounter-1]);
    shrink_Mstringarray(from->ancinputpointer); 
       }
  while(from->systemfilename->arcounter>0) {
    grow_Mstringarray(to->systemfilename, PGSd_PC_FILE_NAME_MAX, from->systemfilename->marray[from->systemfilename->arcounter-1]);
    shrink_Mstringarray(from->systemfilename); 
       }
  while(from->ancsystemfilename->arcounter>0) {
    grow_Mstringarray(to->ancsystemfilename, PGSd_PC_FILE_NAME_MAX, from->ancsystemfilename->marray[from->ancsystemfilename->arcounter-1]);
    shrink_Mstringarray(from->ancsystemfilename); 
       }
  }
else {
  /* just do this... */
  cpy_inputpointer(to, from);
   }
	strcpy(to->rbdt, from->rbdt);
	strcpy(to->redt, from->redt);
	to->orbitnumber=from->orbitnumber;
	to->granulenumber=from->granulenumber;
	strcpy(to->daynightflag,from->daynightflag);
	to->northboundingcoordinate=from->northboundingcoordinate;
	to->southboundingcoordinate=from->southboundingcoordinate;
	to->westboundingcoordinate=from->westboundingcoordinate;
	to->eastboundingcoordinate=from->eastboundingcoordinate;
	for(ig=0;ig<GRINGPOINTS;ig++){
	  to->gringpointlatitude[ig]=from->gringpointlatitude[ig];
	  to->gringpointlongitude[ig]=from->gringpointlongitude[ig];
	  to->gringpointsequenceno[ig]=from->gringpointsequenceno[ig];
	}
	strcpy(to->exclusiongringflag,from->exclusiongringflag);
	strcpy(to->localgranuleid,from->localgranuleid);
	strcpy(to->localgranuleidcoarse,from->localgranuleidcoarse);
	to->rbdt_tai=from->rbdt_tai;
	to->redt_tai=from->redt_tai;
	to->sci_state=from->sci_state;
	to->sci_abnorm=from->sci_abnorm;
	to->int_pgeversion=from->int_pgeversion;
	strcpy(to->equatorcrossingtime, from->equatorcrossingtime);
	strcpy(to->equatorcrossingdate, from->equatorcrossingdate);
	to->equatorcrossinglongitude=from->equatorcrossinglongitude;

	strcpy(to->reprocessingactual, from->reprocessingactual);
	strcpy(to->reprocessingplanned, from->reprocessingplanned);
	strcpy(to->pgeversion, from->pgeversion);
	strcpy(to->processversion, from->processversion);
        for (ig=0;ig<4;ig++)  to->aerosol_clmh[ig] = from->aerosol_clmh[ig];
        to->AutoQualityIsGood=from->AutoQualityIsGood;

}

int32 shrink_Mstringarray(marray_t *string)
/*
!C******************************************************************

!Description: 
            Removes the last entry in a metadata string array.  Returns 1 on failure,
            0 on success.


!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:


Version 2.0, Paul Fisher (SSAI, GSFC Code 923)
1.  Patch to explicitly null string->marray

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************
*/
{

  if(string->arcounter==0)return(1);

  free(string->marray[string->arcounter-1]);
  string->marray[string->arcounter-1]=NULL;
  (string->arcounter)--;
  if(string->arcounter==0){
    free(string->marray);
    string->marray=NULL;
  }

  return(0);
}


void grow_Mstringarray(marray_t *string, int32 length, char *value)
/*
!C******************************************************************

!Description: 
           Grows a metadata string array.



!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************
*/
{

  int i;
  char location[50]="meta.c, grow_Mstringarray\0";

  if(string->arcounter==0){
    string->marray=(char **)CAllocMem(MAX_INPUT_FILES, sizeof(char *));
  }
  else{
    /* don't store duplicates */
    for(i=0;i<string->arcounter;i++){
      if(!(strcmp(string->marray[i], value)))return;
    }
  }

  string->marray[string->arcounter]=
      (char *)AllocMem(length * sizeof(char), location);


  strcpy(string->marray[string->arcounter], value);
  (string->arcounter)++;

  return;
}

void clear_fill(input_metadata *metadata)
/*
!C******************************************************************

!Description: 

!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Jim Ray (SSAI, GSFC Code 923), 24-FEB-00


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Jim Ray
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		jim@cratmos.gsfc.nasa.gov



!Design Notes
    This is a routine to clear the output metadata of all non-string fill values, because the
    products do not insert in downstream databases if their metadata values are out of the 
    expected ranges.

    Code for "correcting" all non-string fill values is here; however, most of it is commented
    out since it is not needed (as of 24-FEB-00).  

!END****************************************************************************
*/
{
char location[50]="meta.c, clear_fill\0";
char comment_format[] = "Metadata field %s was never changed, is set to 0 to avoid metadata insertion problems";
char comment_text[] = "was never changed, is set to 0 to avoid metadata insertion problems";
int ig;

if(metadata->granulenumber == GRANULENUMBERDEFAULT){
      sprintf(msg, comment_format, "GRANULENUMBER");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->granulenumber = 0;
    }
if(metadata->sci_state == SCI_STATE_DEFAULT){
      sprintf(msg, comment_format, "SCI_STATE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->sci_state = 0;
    }
if(metadata->sci_abnorm == SCI_ABNORM_DEFAULT){
      sprintf(msg, comment_format, "SCI_ABNORM");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->sci_abnorm = 0;
    }
if(metadata->equatorcrossinglongitude == EQUATORCROSSINGLONGITUDEDEFAULT){
      sprintf(msg, comment_format, "EQUATORCROSSINGLONGITUDE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->equatorcrossinglongitude = 0.0;
    }
if(metadata->instrumentscansprocessed == INSTRUMENTSCANSPROCESSEDDEFAULT){
      sprintf(msg, comment_format, "INSTRUMENTSCANSPROCESSED");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->instrumentscansprocessed = 0;
    }

/* 
 *  Will comment out all the rest, since fill values in these metadata fields
 *  do not interfere with insertion into any database (at least as far as I know, 
 *  at least as of 24-FEB-00). Jim Ray, SSAI 
 *
if(metadata->northboundingcoordinate == COORDINATEDEFAULT){
      sprintf(msg, comment_format, "NORTHBOUNDINGCOORDINATE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->northboundingcoordinate = 0.0;
    }
if(metadata->southboundingcoordinate == COORDINATEDEFAULT){
      sprintf(msg, comment_format, "SOUTHBOUNDINGCOORDINATE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->southboundingcoordinate = 0.0;
    }
if(metadata->westboundingcoordinate == COORDINATEDEFAULT){
      sprintf(msg, comment_format, "WESTBOUNDINGCOORDINATE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->westboundingcoordinate = 0.0;
    }
if(metadata->eastboundingcoordinate == COORDINATEDEFAULT){
      sprintf(msg, comment_format, "EASTBOUNDINGCOORDINATE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->eastboundingcoordinate = 0.0;
    }

for(ig=0;ig<GRINGPOINTS;ig++){
           if(metadata->gringpointlatitude[ig] == COORDINATEDEFAULT){
              sprintf(msg, "Metadata field GRINGPOINTLATITUDE[%d] %s", ig, comment_text);
              modsmf(MODIS_E_GET_METADATA, msg, location);
              metadata->gringpointlatitude[ig] = 0.0;
           }
           if(metadata->gringpointlongitude[ig] == COORDINATEDEFAULT){
              sprintf(msg, "Metadata field GRINGPOINTLONGITUDE[%d] %s", ig, comment_text);
              modsmf(MODIS_E_GET_METADATA, msg, location);
              metadata->gringpointlongitude[ig] = 0.0;
           }
           if(metadata->gringpointsequenceno[ig] == GRINGPOINTSEQUENCENODEFAULT){
              sprintf(msg, "Metadata field GRINGPOINTSEQUENCENO[%d] %s", ig, comment_text);
              modsmf(MODIS_E_GET_METADATA, msg, location);
              metadata->gringpointsequenceno[ig] = 0;
           }
	}

if(metadata->qapercentother == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTOTHER");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentother = 0;
    }
if(metadata->qapercentnotproducedcloud == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTNOTPRODUCEDCLOUD");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentnotproducedcloud = 0;
    }
if(metadata->qapercentnotproducedother == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTNOTPRODUCEDOTHER");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentnotproducedother = 0;
    }
if(metadata->qapercentgoodqualityfire == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTGOODQUALITYFIRE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentgoodqualityfire = 0;
    }
if(metadata->qapercentotherfire == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTOTHERFIRE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentotherfire = 0;
    }
if(metadata->qapercentnotproducedcloudfire == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTNOTPRODUCEDCLOUDFIRE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentnotproducedcloudfire = 0;
    }
if(metadata->qapercentnotproducedotherfire == MODLANDQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTNOTPRODUCEDOTHERFIRE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentnotproducedotherfire = 0;
    }
if(metadata->percentnotsea == PERCENTNOTSEADEFAULT){
      sprintf(msg, comment_format, "PERCENTNOTSEA");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->percentnotsea = 0;
    }
if(metadata->percentland == PERCENTLANDDEFAULT){
      sprintf(msg, comment_format, "PERCENTLAND");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->percentland = 0;
    }
if(metadata->qapercentoutofbounds == ECSMANDATORYQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTOUTOFBOUNDS");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentoutofbounds = 0;
    }
if(metadata->qapercentmissing == ECSMANDATORYQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTMISSING");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentmissing = 0;
    }
if(metadata->qapercentoutofboundsfire == ECSMANDATORYQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTOUTOFBOUNDSFIRE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentoutofboundsfire = 0;
    }
if(metadata->qapercentmissingfire == ECSMANDATORYQADEFAULT){
      sprintf(msg, comment_format, "QAPERCENTMISSINGFIRE");
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qapercentmissingfire = 0;
    }
*/
/* This is commented out since it is handled more effectively elsewhere (e. g., CLEARFILL).
for(ig=0;ig<BANDPROCESS;ig++){
   if (metadata->qualityclasspercentage[ig][15] == QUALITYCLASSPERCENTAGEDEFAULT) {
      sprintf(msg, "Metadata field QUALITYCLASSPERCENTAGE[%d] %s", ig, comment_text);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      metadata->qualityclasspercentage[ig][15] = 0;
     }
   }
*/

}
  
void split_datetime(char *datetime, char *date, char *time)
/*
!C******************************************************************

!Description: non destructive string parsing, may be slightly slow.


!Input Parameters:



!Output Parameters: 


!Returns:

!Revision History:

Original version
Paul Fisher (SSAI, GSFC Code 923)


!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov



!Design Notes


!END****************************************************************************/
{
  int32 i=0, j=0;
  
  while(datetime[j]!='T'){
    date[i]=datetime[j];
    i++;
    j++;
  }
  date[i]='\0';

  i=0;
  j++;

  while(datetime[j]!='Z'){
    time[i]=datetime[j];
    i++;
    j++;
  }
  time[i]='\0';

  return;

}

void CMGmetinit(char *rbdt)
/*
!C******************************************************************

!Description:   Routine to initialize metadata structures for CMG.

!Input Parameters: none

!Output Parameters:  none

!Input/output Parameters: none

!Returns: none

!Revision History:

Original version
Jim Ray (SSAI, GSFC Code 923)

!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Jim Ray
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		jim@cratmos.gsfc.nasa.gov



!Design Notes
  
      Adapted from ECSmet_init() (above).
      Initializes global variable CMGmetadata.

!END****************************************************************************
*/
{
int j;
char location[50]="meta.c, CMGmetinit\0";

  /* The following is global */
  CMGmetadata=(input_metadata **)AllocMem(NCMGFILES*sizeof(input_metadata *), location);
  
  for(j=0;j<NCMGFILES;j++){
  
    CMGmetadata[j]=(input_metadata *)AllocMem(sizeof(input_metadata), location);
    initECSmetadata(CMGmetadata[j]);
    strcpy(CMGmetadata[j]->rbdt, rbdt);
   
    }
    
}


void adapt_metadata_for_CMG(int type, input_metadata *CMGmetadata, input_metadata *metadata, char *lgid_template)
/*
!C******************************************************************

!Description:   Routine to set metadata values for CMG.  Most values are
                copied from "metadata"; some (all GRING metadata, BOUNDINGCOORDINATE
		metadata, LOCALGRANULEID, etc) are set here.

!Input Parameters: 
     i                      which CMG is being worked on (0=northpole, 1=nonpole, 2=southpole).
     metadata               pointer to the metadata for MOD09/MOD09CRS products.
     lgid_template          character string derived from input localgranuleid of first granule.
     
!Output Parameters: 
     CMGmetadata            pointer to the metadata for this CMG.

!Returns: none

!Revision History:

Original version
Jim Ray (SSAI, GSFC Code 923)

!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Jim Ray
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		jim@cratmos.gsfc.nasa.gov



!Design Notes
     Will undoubtedly have to be revamped once CMG-specific MCFs are devised.

     leftlon   IS GLOBAL!!!!

!END****************************************************************************
*/
{
void CMGmetacpy(input_metadata *to, input_metadata *from);
int i,j;

/* Overwrite what's in the CMG metadata -- EXCEPT for the inputpointer */
CMGmetacpy(CMGmetadata, metadata);

/* Blank out the GRING metadata */
strcpy(CMGmetadata->exclusiongringflag, "N\0");
for (j=0;j<GRINGPOINTS;j++) {
    CMGmetadata->gringpointlatitude[j] = 0.0F;
    CMGmetadata->gringpointlongitude[j] = 0.0F;
    CMGmetadata->gringpointsequenceno[j] = 0;
    }

/* Set the bounding coordinates */
switch (type) {
   case (NTHPL):
      CMGmetadata->northboundingcoordinate = ULLAT;
      CMGmetadata->southboundingcoordinate = POLELIM;
      CMGmetadata->westboundingcoordinate = ULLON;
      CMGmetadata->eastboundingcoordinate = LRLON;
      CMGmetadata->localgranuleidcoarse[7] = 'N';   /* north pole */
      break;
   case (MIDDLE):
      CMGmetadata->northboundingcoordinate = POLELIM;
      CMGmetadata->southboundingcoordinate = -POLELIM;
      CMGmetadata->westboundingcoordinate = (float64)leftlon;
      CMGmetadata->eastboundingcoordinate = (float64)(NONPOLEWIDTH + leftlon);
      if (CMGmetadata->eastboundingcoordinate > 180.0F) 
          CMGmetadata->eastboundingcoordinate -= 360.0F;
      CMGmetadata->localgranuleidcoarse[7] = 'T';   /* "tropics"? */
      break;
   case (STHPL):
      CMGmetadata->northboundingcoordinate = -POLELIM;
      CMGmetadata->southboundingcoordinate = LRLAT;
      CMGmetadata->westboundingcoordinate = ULLON;
      CMGmetadata->eastboundingcoordinate = LRLON;
      CMGmetadata->localgranuleidcoarse[7] = 'S';   /* south pole */
      break;
   }

/* Set the rest of the local granule id */
CMGmetadata->localgranuleidcoarse[5] = 'I';
CMGmetadata->localgranuleidcoarse[6] = 'D';
if (CMGmetadata->inputpointer->arcounter){
    /* Old method took localgranuleid time from 1st granule in the CMG file, so it (usually) 
       differed for each CMG file.  This is no good, since the CMG, as an orbital product, 
       needs to have a LGID time equal to the start time of the orbit, regardless of the CMG
       contents.        
    for (j=18;j<22;j++)
       CMGmetadata->localgranuleidcoarse[j] = CMGmetadata->inputpointer->marray[0][j];
       See below for new method.
     */
    }
else /* this shouldn't happen but if it does... */
   CMGmetadata->inputpointer->marray = NULL;
   
/* 22-MAY-02: lgid_template is a string derived from the LOCALGRANULEID of one (any) 
   of the L1B inputs for the first granule.  By "derived" I mean minus the ESDT,
   so it begins with an A.  Simply copy characters 0-12 of lgid_template to
   characters 9-21 of CMGmetadata->localgranuleidcoarse. */
for (j=9,i=0;j<=21;j++,i++)
   CMGmetadata->localgranuleidcoarse[j] = lgid_template[i];
          
}

void CMGmetacpy(input_metadata *to, input_metadata *from)
/*
!C******************************************************************

!Description:   Routine to copy SOME metadata values from "from", to "to".

!Input Parameters: 
     from                    pointer to the metadata for MOD09/MOD09CRS products.

!Output Parameters: 
     to                      pointer to the metadata for the CMG.
           
!Returns: none

!Revision History:

Original version
Jim Ray (SSAI, GSFC Code 923)

!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Jim Ray
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		jim@cratmos.gsfc.nasa.gov



!Design Notes
      Order in which metadata elements are listed is the same as in proc_struct.h, just
      to keep things clear.

!END****************************************************************************
*/
{
int i,j;

/*
Will skip (for now)
  marray_t *inputpointer;
  marray_t *systemfilename;
  marray_t *ancinputpointer;
  marray_t *ancsystemfilename;
  marray_t *inputpointer_fire; 
  marray_t *systemfilename_fire; 
  marray_t *ancinputpointer_fire; 
  marray_t *ancsystemfilename_fire; 
*/

/*
Will skip (for now)
  int32 *last_filename;
  int32 last_filename_index;
*/

/* This will be taken care of elsewhere...
strcpy(to->rbdt, from->rbdt);*/
strcpy(to->redt, from->redt);
to->orbitnumber = from->orbitnumber;
to->granulenumber = from->granulenumber;
strcpy(to->daynightflag, from->daynightflag);
strcpy(to->daynightflag, "Day\0");   /* permissible? */

/* 
Will ignore all BOUNDING COORDINATES here, since they will be set later
  float64 northboundingcoordinate;
  float64 southboundingcoordinate;
  float64 westboundingcoordinate;
  float64 eastboundingcoordinate;
*/
/*
Will ignore all GRING stuff here, since they will be set later
  float64 gringpointlatitude[GRINGPOINTS];
  float64 gringpointlongitude[GRINGPOINTS];
  int32 gringpointsequenceno[GRINGPOINTS];
  char exclusiongringflag[PGSd_PC_VALUE_LENGTH_MAX];
*/

to->instrumentscansprocessed = from->instrumentscansprocessed;
to->sci_state = from->sci_state;
to->sci_abnorm = from->sci_abnorm;
strcpy(to->reprocessingplanned, from->reprocessingplanned);
strcpy(to->reprocessingactual, from->reprocessingactual);
strcpy(to->seaprocessed, from->seaprocessed);
strcpy(to->adjacencycorrectionattempted, from->adjacencycorrectionattempted);
strcpy(to->atmosphericcorrectionattempted, from->atmosphericcorrectionattempted);
strcpy(to->attributenameqapercentgoodquality, from->attributenameqapercentgoodquality);
strcpy(to->attributenameqapercentotherquality, from->attributenameqapercentotherquality);
strcpy(to->attributenameqapercentnotproducedcloud, from->attributenameqapercentnotproducedcloud);
strcpy(to->attributenameqapercentnotproducedother, from->attributenameqapercentnotproducedother);
strcpy(to->attributenameqapercentpooroutput250mband1, from->attributenameqapercentpooroutput250mband1);
strcpy(to->attributenameqapercentpooroutput250mband2, from->attributenameqapercentpooroutput250mband2);
strcpy(to->attributenameqapercentpooroutput500mband1, from->attributenameqapercentpooroutput500mband1);
strcpy(to->attributenameqapercentpooroutput500mband2, from->attributenameqapercentpooroutput500mband2);
strcpy(to->attributenameqapercentpooroutput500mband3, from->attributenameqapercentpooroutput500mband3);
strcpy(to->attributenameqapercentpooroutput500mband4, from->attributenameqapercentpooroutput500mband4);
strcpy(to->attributenameqapercentpooroutput500mband5, from->attributenameqapercentpooroutput500mband5);
strcpy(to->attributenameqapercentpooroutput500mband6, from->attributenameqapercentpooroutput500mband6);
strcpy(to->attributenameqapercentpooroutput500mband7, from->attributenameqapercentpooroutput500mband7);
strcpy(to->attributenameqapercentpooroutput1kmband1, from->attributenameqapercentpooroutput1kmband1);
strcpy(to->attributenameqapercentpooroutput1kmband2, from->attributenameqapercentpooroutput1kmband2);
strcpy(to->attributenameqapercentpooroutput1kmband3, from->attributenameqapercentpooroutput1kmband3);
strcpy(to->attributenameqapercentpooroutput1kmband4, from->attributenameqapercentpooroutput1kmband4);
strcpy(to->attributenameqapercentpooroutput1kmband5, from->attributenameqapercentpooroutput1kmband5);
strcpy(to->attributenameqapercentpooroutput1kmband6, from->attributenameqapercentpooroutput1kmband6);
strcpy(to->attributenameqapercentpooroutput1kmband7, from->attributenameqapercentpooroutput1kmband7);
strcpy(to->attributenamegranulenumber, from->attributenamegranulenumber);
strcpy(to->brdfcorrectionattempted, from->brdfcorrectionattempted);

for (i=0;i<BANDPROCESS;i++) {
    for (j=0;j<16;j++)to->qualityclasspercentage[i][j] = from->qualityclasspercentage[i][j];
  }
to->qapercentgoodquality = from->qapercentgoodquality;
to->qapercentother = from->qapercentother;
to->qapercentnotproducedcloud = from->qapercentnotproducedcloud;
to->qapercentnotproducedother = from->qapercentnotproducedother;
to->qapercentoutofbounds = from->qapercentoutofbounds;
to->qapercentmissing = from->qapercentmissing;
to->percentnotsea = from->percentnotsea;
to->percentland = from->percentland;
strcpy(to->localgranuleidversion, from->localgranuleidversion);

/* 
Will skip (for now)
  char localgranuleidversionfire[PGSd_PC_VALUE_LENGTH_MAX];
  char localgranuleid[PGSd_PC_VALUE_LENGTH_MAX];
  char localgranuleidfire[PGSd_PC_VALUE_LENGTH_MAX];
*/

strcpy(to->localgranuleidcoarse, from->localgranuleidcoarse);
strcpy(to->pgeversion, from->pgeversion);
strcpy(to->autoquality, from->autoquality);

/* 
Will skip (for now)
  char autoquality_fire[PGSd_PC_VALUE_LENGTH_MAX];
  int32 qapercentoutofboundsfire;
  int32 qapercentmissingfire;
  int32 qapercentgoodqualityfire;
  int32 qapercentotherfire;
  int32 qapercentnotproducedcloudfire;
  int32 cloudpixelsfire;
  int32 numwaterpixelsfire;
  int32 numunknownpixelsfire;
  int32 numnofirepixelsfire;
  int32 numglintpixelsfire;
  int32 numcloudadjacentfire;
  int32 numwateradjacentfire;
  int32 qapercentnotproducedotherfire;
  int32 firepixels;
  char attributenamefirepixels[PGSd_PC_VALUE_LENGTH_MAX];
*/

strcpy(to->equatorcrossingdate, from->equatorcrossingdate);
strcpy(to->equatorcrossingtime, from->equatorcrossingtime);
to->equatorcrossinglongitude = from->equatorcrossinglongitude;
to->highsolarzenith = from->highsolarzenith;
for (i=0;i<4;i++)  to->aerosol_clmh[i] = from->aerosol_clmh[i];
strcpy(to->attributenameprocessversion, from->attributenameprocessversion);
strcpy(to->processversion, from->processversion);

/* 
Will skip (for now)
  aq_t AutoQualityIsGood;
*/

strcpy(to->used_int_cloud_mask, from->used_int_cloud_mask);

}


void read_dao_metadata(PGSt_PC_Logical file_lun, char *rbd, d_ancillary *dao)
/*
!C*****************************************************************************

!Description: reads RANGEBEGINNINGDATE from dao files.  If the date is acceptable, 
              inputs date and version number into the d_ancillary structure for
              use later.

!Input Parameters:
file_lun
rbd

!Output Parameters:
none

!Input/Ouput Parameters:
dao

!Returns:
none

!Revision History:
none

Revision 1.0, P. Fisher, Jan. 1998
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-31365

!References and Credits:
  
    MODIS Science Team Members:
      Eric. Vermote
      MODIS Land Science Team           University of Maryland, College Park
      eric@kratmos.gsfc.nasa.gov        NASA's GSFC Code 923
      phone: 301-286-6232               Greenbelt, MD 20771

    Developers:
      Paul Fisher
      MODIS Land Science Team           Science Systems and Applications Inc.
      fisher@ltpmail.gsfc.nasa.gov      NASA's GSFC Code 923 
      phone (301) 286-2980              Greenbelt, MD 20771
 
!Design Notes

!END
****************************************************************************/
{

  char location[50]="meta.c, read_dao_metadata\0";
  PGSt_integer i;
  int k;
  PGSt_double juliangran[2], juliandao[2];
  int jgran, jdao;
  char string_string[PGSd_PC_VALUE_LENGTH_MAX];
  	   
  if(PGS_PC_GetNumberOfFiles(file_lun, &i)!=PGS_S_SUCCESS){
    sprintf(msg, "%d", (int)file_lun);
    modsmf(MODIS_E_GETNUM_FILES, msg, location);
    return;
  }
  
  /* convert date of the first granule to time code b */
  if(PGS_TD_UTCtoUTCjd(rbd, juliangran)!=PGS_S_SUCCESS){
    modsmf(MODIS_E_BAD_TIME_CONV, "Unable to convert date/time of first granule to julian date.  "
	   "Unable to process DAO data.", location);
    return;
  }
  jgran=(int)juliangran[0];

  for(k=1;k<=(int)i;k++){

    if(readECSmetadata_string(file_lun, k, CORES, Mfield_name[RANGEBEGINNINGDATE], string_string)!=
       MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      Mfield_name[RANGEBEGINNINGDATE], (LINT)file_lun, (int32)k);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      continue;
    }

    if(PGS_TD_UTCtoUTCjd(string_string, juliandao)!=PGS_S_SUCCESS){
      sprintf(msg, "Unable to convert date of first granule to julian date.  "
	      "Unable to process DAO data for LUN %d, version %d.", (int)file_lun, (int)k);
      modsmf(MODIS_E_BAD_TIME_CONV, msg, location);
      continue;
    }
    jdao=(int)juliandao[0];

    if(jdao==jgran || jdao==jgran+1){
      dao->jrbd[dao->numdays]=(LINT)jdao;
      dao->versions[dao->numdays]=k;
      dao->numdays++;
    }
      
  }/* for k... */

  return;
}


void read_ancillary_metadata(int mode, char *rbd, d_ancillary *dao, char *logreport_string)
/*
!C*****************************************************************************

!Description: reads RANGEBEGINNINGDATE from dao or grib files.  If the date is 
              acceptable, inputs date and version number into the d_ancillary 
              structure for use later.

!Input Parameters:
rbd

!Output Parameters:
none

!Input/Ouput Parameters:
dao

!Returns:
none

!Revision History:
1.1: 16-JUN-99, derived from read_dao_metadata() (above), to allow reads from 
                GRIB-format files 

Revision 1.0, P. Fisher, Jan. 1998
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-31365

!References and Credits:
  
    MODIS Science Team Members:
      Eric. Vermote
      MODIS Land Science Team           University of Maryland, College Park
      eric@kratmos.gsfc.nasa.gov        NASA's GSFC Code 923
      phone: 301-286-6232               Greenbelt, MD 20771

    Developers:
      Paul Fisher
      MODIS Land Science Team           Science Systems and Applications Inc.
      fisher@ltpmail.gsfc.nasa.gov      NASA's GSFC Code 923 
      phone (301) 286-2980              Greenbelt, MD 20771
 
      Jim Ray
      MODIS Land Science Team           Science Systems and Applications Inc.
      jim@cratmos.gsfc.nasa.gov         NASA's GSFC Code 923 
      phone (301) 614-6684              Greenbelt, MD 20771

!Design Notes

!END
****************************************************************************/
{

  char location[50]="meta.c, read_ancillary_metadata\0";
  PGSt_integer daoi, gribi, ver;
  PGSt_integer dao_index, grib_index;
  int i,alt_OK,k,next, same;
  float tmpflt;
  PGSt_double juliangran[2], s_juliandao[2], e_juliandao[2];
  PGSt_double real_gran_time, real_anci_time, real_sanci_time, real_eanci_time, difference;
  int jgran, jdao, start;
  char string_string[PGSd_PC_VALUE_LENGTH_MAX]={'\0'};
  char time_string[PGSd_PC_VALUE_LENGTH_MAX]={'\0'};
  
  /* variables for reading from GRIB files */
  PGSt_double grib_time_resolution;
  char tf_name[PGSd_PC_FILE_PATH_MAX]={'\0'};
  FILE *fd;   
  char where[25], dao_newname[10], grib_newname[10];
  int read_grib_date(FILE *input, char *what, char *where, char *date); 
  int read_EPT_date(FILE *input, char *date); 
  char date_and_time[PGSd_PC_FILE_PATH_MAX]={'\0'};

  alt_OK=daoi=gribi=0;
  
  /* figure out what we are processing */   
  if (!strcmp(dao->tag, OZZONE) || !strcmp(dao->tag, OZONE_GRIBTAG)) {  
      dao_index = DAOOZONE; 
      strcpy(dao_newname, DAOOZONEVARIABLE);
      grib_index = EPT_OZ; 
      strcpy(grib_newname, OZONE_GRIBTAG);  
      grib_time_resolution = (PGSt_double)EPT_OZONE_TIMERES;
      alt_OK = 1;
      }
  else if (!strcmp(dao->tag, GOZZONE) || !strcmp(dao->tag, DAOOZONEVARIABLE) ||
           !strcmp(dao->tag, GOZONE_GRIBTAG)) {  
      dao_index = DAOWV; 
      strcpy(dao_newname, DAOOZONEVARIABLE);
      grib_index = GRIB_WV; 
      strcpy(grib_newname, GOZONE_GRIBTAG);  
      strcpy(where, OZONE_GRIBLEVEL);
      grib_time_resolution = (PGSt_double)GRIB_WV_SP_TIMERES;
      }
  else if (!strcmp(dao->tag, WATERV) || !strcmp(dao->tag, DAOWVVARIABLE) ||
           !strcmp(dao->tag, WV_GRIBTAG)) {
      dao_index = DAOWV; 
      strcpy(dao_newname, DAOWVVARIABLE);
      grib_index = GRIB_WV; 
      strcpy(grib_newname, WV_GRIBTAG);
      strcpy(where, WV_GRIBLEVEL);
      grib_time_resolution = (PGSt_double)GRIB_WV_SP_TIMERES;
      }
  else if (!strcmp(dao->tag, SURFPR) || !strcmp(dao->tag, DAOSPRESVARIABLE) ||
           !strcmp(dao->tag, SPRES_GRIBTAG)) {
      dao_index = DAOSPRES; 
      strcpy(dao_newname, DAOSPRESVARIABLE);
      grib_index = GRIB_SP; 
      strcpy(grib_newname, SPRES_GRIBTAG);
      strcpy(where, SPRES_GRIBLEVEL);
      grib_time_resolution = (PGSt_double)GRIB_WV_SP_TIMERES;
      }
  else if (!strcmp(dao->tag, TEMPR) || (!strcmp(dao->tag, TMP_GRIBTAG))) {
      dao_index = DAOSPRES;                   /* necessary? */
      strcpy(dao_newname, DAOSPRESVARIABLE);  /* necessary? */
      grib_index = GRIB_WV;    /* can be the same since temp data is (currently) in the same file as WV */
      strcpy(grib_newname, TMP_GRIBTAG);
      strcpy(where, TMP_GRIBLEVEL);
      grib_time_resolution = (PGSt_double)GRIB_WV_SP_TIMERES;  /* same file, same time resolution */
      }
  else if (!strcmp(dao->tag, UWND) || (!strcmp(dao->tag, UWD_GRIBTAG))) {
      dao_index = DAOSPRES;                   /* necessary? */
      strcpy(dao_newname, DAOSPRESVARIABLE);  /* necessary? */
      grib_index = GRIB_WV;    /* can be the same since UWND data is (currently) in the same file as WV */
      strcpy(grib_newname, UWD_GRIBTAG);
      strcpy(where, UWD_GRIBLEVEL);
      grib_time_resolution = (PGSt_double)GRIB_WV_SP_TIMERES;  /* same file, same time resolution */
      }
  else if (!strcmp(dao->tag, VWND) || (!strcmp(dao->tag, VWD_GRIBTAG))) {
      dao_index = DAOSPRES;                   /* necessary? */
      strcpy(dao_newname, DAOSPRESVARIABLE);  /* necessary? */
      grib_index = GRIB_WV;    /* can be the same since UWND data is (currently) in the same file as WV */
      strcpy(grib_newname, VWD_GRIBTAG);
      strcpy(where, VWD_GRIBLEVEL);
      grib_time_resolution = (PGSt_double)GRIB_WV_SP_TIMERES;  /* same file, same time resolution */
      }
  else {
      sprintf(msg, " unexpected ancillary data file tag: '%s'", dao->tag);
      modsmf(MODIS_F_CODING_ERROR, msg, location);
      }

  /* Get number of versions of each type; first, dao... *   /
   *
   * No, will disable DAO processing for now (30-JUN-01)
  if(PGS_PC_GetNumberOfFiles(dao_index, &daoi)!=PGS_S_SUCCESS){
    sprintf(msg, "%d, DAO ancillary data", (int)dao_index);
    modsmf(MODIS_E_GETNUM_FILES, msg, location);
  }
   */

  /* ... then grib */
  if(PGS_PC_GetNumberOfFiles(grib_index, &gribi)!=PGS_S_SUCCESS){
    sprintf(msg, "%d, GRIB ancillary data", (int)grib_index);
    modsmf(MODIS_E_GETNUM_FILES, msg, location);
  }
  

  if ((daoi == 0)&&(gribi == 0)) return;
  
  /* new, 11-JAN-2002 */
  if (gribi == 1) {
      sprintf(msg, ": there is only one NCEP file with '%s' in the PCF", dao->tag);
      modsmf(MODIS_F_ANCILLARY_DATA_ERROR, msg, location);
      }
  /* end, 11-JAN-2002 */
  
  dao->numdays = 0;                      /* set this here, since we currently check the versions 
                                            each time the granule changes. */
					     
  /* convert date of the first granule to time code b */
  juliangran[0] = juliangran[1] = 0.0;
  if(PGS_TD_UTCtoUTCjd(rbd, juliangran)!=PGS_S_SUCCESS){
    modsmf(MODIS_E_BAD_TIME_CONV, "Unable to convert date/time of first granule to julian date.  "
	   "Unable to process ancillary data.", location);
    return;
  }
  real_gran_time = (juliangran[0] - 0.5) + juliangran[1];
  jgran=(int)juliangran[0];

/*printf("%p %s %d %f %f\n", juliangran, rbd, PGS_TD_UTCtoUTCjd(rbd, juliangran), juliangran[0], juliangran[1] );*/

  /* try to get the data from GRIB files first */
      for(k=1;k<=(int)gribi;k++) {
         /* get file name */
         ver = k;
         if(PGS_PC_GetReference(grib_index, &ver, tf_name)!=PGS_S_SUCCESS){
             sprintf(msg, "can't get filename, LUN %d, version %d", (int32)grib_index, k);
             modsmf(MODIS_E_GET_METADATA, msg, location);
             continue;
	        }
         else{  /* open file */
             if ((fd=fopen(tf_name,"rb")) == NULL) {
                sprintf(msg, "can't open file, LUN %d, version %d", (int32)grib_index, k);
                modsmf(MODIS_E_GET_METADATA, msg, location);
                continue;
	        }
             else{ /* get date from file */
	     
                string_string[0] = '\0';  /* 11-OCT-04: Very important */
		
                if (alt_OK) { /* 29-APR-04: Then you are reading OZONE.  USE EP-TOMS 
		                  reading routines... */
	           read_EPT_date(fd, string_string);  
		   }
                else
	           read_grib_date(fd, grib_newname, where, string_string);
	        /*printf("%s\n*** %s %s ***\n", location, grib_newname, string_string);*/
	

	        fclose(fd);
	        if (string_string[0] == '\0'){
                    sprintf(msg, "can't get date, CHECK FILE: LUN %d, version %d", 
		            (int32)grib_index, k);
                    modsmf(MODIS_E_GET_METADATA, msg, location);
                    continue;
	           }

                if(PGS_TD_UTCtoUTCjd(string_string, s_juliandao)!=PGS_S_SUCCESS){
                   sprintf(msg, "Unable to convert date of first granule to julian date.  "
	                        "Unable to process NCEP data for LUN %d, version %d.", 
			        (int)dao_index, (int)k);
                   modsmf(MODIS_E_BAD_TIME_CONV, msg, location);
                   continue;
                  }
                real_anci_time = (s_juliandao[0] - 0.5) + s_juliandao[1];
		difference = real_gran_time-real_anci_time;
		if (difference < 0.00000)  difference = -difference;
                jdao=(int)s_juliandao[0];

                /* ASSUMING grib data comes every six hours (0, 6, 12, 18) for WV/SP/etc,
		   24 hr for ozone (see proc_define.h), if difference is less than xxx
		   hrs, take it. */

                if (difference < (grib_time_resolution/24.0)) {  

		   /*
		   printf("META: %s\n%s\nGranule  time: %s OR %lf, %d\n",tf_name,grib_newname, rbd, real_gran_time, jgran);
		   printf("Metadata time: %s OR %lf, %d\n",string_string, real_anci_time, jdao);
		   printf("dao->numdays = %d, k = %d\n",dao->numdays, k);
		   printf("weight = %f, res = %f\n",difference, (grib_time_resolution/24.0));
                   */
		   /* Added here; update the global variable "metadata" */
		   if /*(mode == 0)*/ (mode != 1) {
		      if(getECSfileinfo(grib_index, k, metadata)!=MODIS_S_SUCCESS){   /* metadata is global */
                          sprintf(msg, "LUN %d, version %d", (int32)grib_index, k);
                          modsmf(MODIS_F_GETFILEINFO, msg, location);
                           }
		    }
		    
		     /* if (strcmp(dao->anci_source, "GRIB\0")) {
		          dao->numdays = 0;
			  dao->lastversion = -1;   / * just so program will read the GRIB 
			                              data if it used DAO data with the same 
						      version number the last time. * /
			  }*/
                      dao->jrbd[dao->numdays]=(LINT)jdao;  /* still necessary? */
                      dao->versions[dao->numdays]=k;
                      dao->weights[dao->numdays]=difference;
                      dao->numdays++; 
                      strcpy(dao->tag, grib_newname);
                      if (alt_OK) strcpy(dao->anci_source, "EPT\0");
                      else        strcpy(dao->anci_source, "GRIB\0");
		      }  /* if (difference < ((grib_... */
                   }
                }
	     
                                 }  /* for k... */

        /* 08-MAY-01: make sure we have two times */
        if ((dao->numdays < 1)||(dao->numdays > 2)) {
            sprintf(msg, "('%s' NCEP data)", dao->tag);
            modsmf(MODIS_F_ANCILLARY_DATA_ERROR, msg, location);
        }

        if (dao->numdays == 1) {
            /* Then in all likelihood you are at the times of 00, 06, 12 or 18, times that 
	       are on the time-borders of ancillary data files.  However, these is still a 
	       chance that the wrong files were staged.  Check the weight value; it is
	       zero ONLY at ancillary data file time-borders: */
	       
	       if ( fabsf(dao->weights[dao->numdays-1]) < 0.001F )  {
	           /* then we're OK; simply duplicate the file and set 
		      the duplicates' weight to the right value.   */
/*printf("**********************\n");
printf("Duplicating: granule time: %s %f\n", rbd, dao->weights[dao->numdays-1]);
printf("**********************\n");*/
                   dao->versions[dao->numdays]=dao->versions[dao->numdays-1];
                   dao->weights[dao->numdays]=(grib_time_resolution/24.0);
                   dao->numdays++; 
                   strcpy(dao->tag, grib_newname);
                   if (alt_OK) strcpy(dao->anci_source, "EPT\0");
                   else        strcpy(dao->anci_source, "GRIB\0");
	       }
	       else {
	           /* complain! */
                   sprintf(msg, ", check the PCF: '%s' NCEP data", dao->tag);
                   modsmf(MODIS_F_ANCILLARY_DATA_ERROR, msg, location);
	       }
	} 

        /* 03-MAY-01: check, swap the weights */
        if (dao->numdays == 2) {
	   if ( fabsf((dao->weights[0]+dao->weights[1]) - (grib_time_resolution/24.0)) > 0.1F) {
	        /* This, incidentally, should never happen */
                sprintf(msg, "Unable to apply weights to %s NCEP data sets of different times."
		             " Will take NCEP data of version %d.", dao->tag, dao->versions[0]);
                dao->numdays=1;
                modsmf(MODIS_E_GET_METADATA, msg, location);
		}
	   else {
	       tmpflt = dao->weights[0]*(24.0F/grib_time_resolution);
	       dao->weights[0] = dao->weights[1]*(24.0F/grib_time_resolution);
	       dao->weights[1] = tmpflt;
	   }
	   
 	}
         dao->numdays--;
	 
	if (dao->numdays > -1) {
	   if (strlen(logreport_string) == 0) sprintf(logreport_string, "And...\n\0");
           for(k=0;k<=dao->numdays;k++) {
               ver=(PGSt_integer)dao->versions[k];
               PGS_PC_GetReference(grib_index, &ver, tf_name);
               /*printf("***** File %s, weight %f\n", tf_name,
	       dao->weights[k]);*/
	       if (!strcmp(grib_newname, "PWAT") || !strcmp(grib_newname, "TOZNE"))
	          sprintf(logreport_string+strlen(logreport_string), 
	               "LUN %d, version %d, file %s, weight %f\n\0",
		       grib_index, dao->versions[k], tf_name, dao->weights[k]);
           }
        }

  if (strcmp(dao->tag, grib_newname)) {  /* then the GRIB has failed, try DAO */
  for(k=1;k<=(int)daoi;k++) {

    /* 22-JUN-99: Since these files can begin either at 2100 the day before *OR* at
       0000 of the current day, I thought I'd make figuring which file we need as 
       explicit as possible.  See routine "concatenate_fields()", below. Jim Ray, SSAI */
    concatenate_fields(dao_index, k, Mfield_name[RANGEBEGINNINGDATE], 
          Mfield_name[RANGEBEGINNINGTIME], date_and_time);
    if (date_and_time == NULL) continue;
    strcpy(string_string, date_and_time);
    
    if(PGS_TD_UTCtoUTCjd(string_string, s_juliandao)!=PGS_S_SUCCESS){
      sprintf(msg, "Unable to convert start date of DAO ancillary data file to julian date.  "
	      "Unable to process DAO data for LUN %d, version %d.", (int)dao_index, (int)k);
      modsmf(MODIS_E_BAD_TIME_CONV, msg, location);
      continue;
    }
    
    concatenate_fields(dao_index, k, Mfield_name[RANGEENDINGDATE], 
          Mfield_name[RANGEENDINGTIME], date_and_time);
    if (date_and_time == NULL) continue;
    strcpy(string_string, date_and_time);

    if(PGS_TD_UTCtoUTCjd(string_string, e_juliandao)!=PGS_S_SUCCESS){
      sprintf(msg, "Unable to convert end date of DAO ancillary data file to julian date.  "
	      "Unable to process DAO data for LUN %d, version %d.", (int)dao_index, (int)k);
      modsmf(MODIS_E_BAD_TIME_CONV, msg, location);
      continue;
    }
    
    real_sanci_time = (s_juliandao[0] - 0.5) + s_juliandao[1];
    real_eanci_time = (e_juliandao[0] - 0.5) + e_juliandao[1] + 0.125;
    /* 21-JUN-00: This here                                   ^^^^^^^
       because although end time is typically 21 hr, the coverage is until 24 hr.  The RANGEENDINGTIME
       metadata does not reflect actual coverage of the data, at least as of today. */

    jdao = (int)(s_juliandao[1]*24.0);  /* quite different!!! */

    if((real_sanci_time <= real_gran_time) &&
       (real_eanci_time >= real_gran_time)) {   /* SUCCESS!! */
      dao->jrbd[dao->numdays]=(LINT)jdao;    
      dao->versions[dao->numdays]=k;
      dao->numdays++;
      strcpy(dao->tag, dao_newname);
      strcpy(dao->anci_source, "DAO\0");
    }
      
                             }  /* for k... */
	                             }  /* if (strcmp(da... */


/*
 * printf("Granule time = %s, Tag = '%s', versions: %d, %d, weights: %f, %f\n", 
 *     rbd, dao->tag, dao->versions[0],dao->versions[1], dao->weights[0],dao->weights[1]);
 */
 
  return;
}

int read_EPT_date(FILE *input, char *date)
/*
!C**********************************************************************
!Description:   
  The first few lines of the ascii TOMS file look like:

 Day: 272 Sep 29, 1997    EP/TOMS    NRT OZONE    GEN:97.273 Asc LECT: 11:19 AM 
 Longitudes:  288 bins centered on 179.375 W to 179.375 E  (1.25 degree steps)  
 Latitudes :  180 bins centered on  89.5   S to  89.5   N  (1.00 degree steps)  

 This routine puts together the date from line 1.
 
!Input Parameters:
  FILE *input -- File pointer (return value of fopen()).

!Output Parameters:
  char *date  -- date in ASCII time code A (28 characters) (e.g., 2000-09-14T08:30:00.000000Z)

Global Variables:

!Revision History:
  1997/10/?? Neal Devine -- Initial Version
  
  Adapted from MOD_PROZN code, routine Read_TOMS_Ozone(), on 29-APR-04 by Jim Ray (SSAI).

!Team-unique Header:
  !References and Credits:
     Neal Devine (neal.devine@gsfc.nasa.gov)

     This software is developed by the MODIS Science Data Support
     Team for the National Aeronautics and Space Administration,
     Goddard Space Flight Center, under contract NAS5-32373.

     HDF portions developed at the National Center for Supercomputing
     Applications at the University of Illinois at Urbana-Champaign.

!Design Notes: 
   
PDL:
!END********************************************************************
*/
{
#define MAX_LINE  333
#define NLAT_MAX  181
#define NLON_MAX  360
  char line[MAX_LINE], month[4];
  int i, nlat, nlon, yday, year, dom, m;
  char msg[256];
  char location[50]="meta.c, read_EPT_date\0";
  char months[12][4] = {"JAN\0","FEB\0","MAR\0","APR\0","MAY\0","JUN\0","JUL\0","AUG\0","SEP\0","OCT\0","NOV\0","DEC\0"};

  /* Read first three lines of TOMS Ozone file, get the following:
   *  line 1) day & year,
   * While we're at it may as well check the following:
   *  line 2) # longitude bins.
   *  line 3) # latitude bins.
   */
  for ( i = 0 ; i < 3 ; i++ ) {
    if ( fgets(line, MAX_LINE, input) == NULL ) {
      sprintf(msg, "failed to read line %d of EP-TOMS Ozone file", i);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return( 1 );
    }
    switch (i) {
    case 0 : sscanf( line, "%*s %d %s %d, %d", &yday, month, &dom, &year ); break;
    case 1 : sscanf( line, "%*s %d", &nlon ); break;
    case 2 : sscanf( line, "%*s %*s %d" , &nlat ); break;
      /*    case 0 : sscanf( line, "Day: %d %*s %*d, %d", yday, year ); break;*/
      /*    case 1 : sscanf( line, "Longitudes: %d", nlon ); break;*/
      /*    case 2 : sscanf( line, "Latitudes : %d" , nlat ); break;*/
    }
  }

  if( ((nlat*nlon) > (NLAT_MAX*NLON_MAX)) || ( (nlat*nlon) <= 0) ) {
      sprintf(msg, "EP-TOMS Ozone file has unexpected numbers of latitude or longitude bins, %d and %d", nlat, nlon);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return( 2 );
     }

  if (year < 70) year += 2000;
  else if (year < 100) year += 1900;   /* just in case these EPTOMS files somehow get 2-digit years in them! */

  for(i=0;i<3;i++) month[i] = toupper(month[i]);

  m = -1;  
  for(i=0;i<12;i++) if (!strcmp(month, months[i])) m = i;
  if ( m < 0 ) {
      sprintf(msg, "EP-TOMS Ozone file has unexpected month value in header, '%s' -- cannot continue", month);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return( 2 );
     }

  m++;  
  sprintf(date, "%4d-%02d-%02dT12:00:00.000000Z", year, m, dom);

  return( 0 );
}


void concatenate_fields(PGSt_PC_Logical file_lun, int k, char *field1, char *field2, char *d_and_t)
/*
!C*****************************************************************************

!Description: reads two core metadata fields -- date and time --  from dao ancillary 
              data files, and concatenates them to form one 28-character ASCII_TCA-
	      formatted date.

!Input Parameters:
file_lun      which file to read from, and
k             which version number
field1        name of the first field to get -- must be either the RANGEBEGINNINGDATE
              or RANGEENDINGDATE field         
field2        name of the second field to get -- must be either the RANGEBEGINNINGTIME
              or RANGEENDINGTIME field         
              Neither field is actually checked, though.
	      
!Output Parameters:
d_and_t       date and time, concatenated

!Input/Ouput Parameters:
none

!Returns:
none

!Revision History:

Revision 1.0, J. Ray, Jun. 1999
1. Original Version

!Team-unique Header:

    This software developed by the MODIS Land Science Team for
    the National Aeronautics and Space Administration, Goddard
    Space Flight Center, under contract NAS5-31365

!References and Credits:
  
    MODIS Science Team Members:
      Eric. Vermote
      MODIS Land Science Team           University of Maryland, College Park
      eric@kratmos.gsfc.nasa.gov        NASA's GSFC Code 923
      phone: 301-286-6232               Greenbelt, MD 20771

    Developers:
      Jim Ray
      MODIS Land Science Team           Science Systems and Applications Inc.
      jim@cratmos.gsfc.nasa.gov         NASA's GSFC Code 923 
      phone (301) 614-6684              Greenbelt, MD 20771

!Design Notes
     

!END
****************************************************************************/
{
  char location[50]="meta.c, concatenate_fields\0";
  int i, start;
  char string_string[PGSd_PC_VALUE_LENGTH_MAX]={'\0'};
  char time_string[PGSd_PC_VALUE_LENGTH_MAX]={'\0'};

    if(readECSmetadata_string(file_lun, k, CORES, field1, string_string)!= MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      field1, (LINT)file_lun, (int32)k);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return;
    }
    if(readECSmetadata_string(file_lun, k, CORES, field2, time_string)!= MODIS_S_SUCCESS){
      sprintf(msg, "%s metadata field was not found within the file with lun %ld and version %d", 
	      field2, (LINT)file_lun, (int32)k);
      modsmf(MODIS_E_GET_METADATA, msg, location);
      return;
    }
    
    start = strlen(time_string);
    time_string[start-1] = '.';
    for (i=0;i<6;i++)  time_string[start+i] = '0';
    strcat(time_string,"Z\0");
    sprintf(d_and_t, "%sT%s", string_string, time_string);
}


int32 getjulianday(char *timecodea)
/*
!C******************************************************************

!Description: extracts the julian day of the year from an input 
              CCSDS ascii time code A string.


!Input Parameters:
timecodea:  string containing a CCSDS time code A

!Output Parameters: none

!Returns: the julian day of year as a signed int32

!Revision History:
Version 1.1 April 2, 1997, P. Fisher/SSAI/GSFC Code 923
Integration into actual processing code.

!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062



Version 1.0 April 1, 1997, A. Vermeulen/UMd/GSFC Code 923
Original prototype


!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771
			eric@kratmos.gsfc.nasa.gov
			301-286-6232

DEVELOPERS:
                Anne Vermeulen
		Univerisity of MD.
		NASA's GSFC Code 923
		Greenbelt, MD , 20771
		anne@kratmos.gsfc.nasa.gov

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

!Design Notes:

This module does not check for the consistency of the input time code
A string.  It is assumed that this is performed by the calling
function.

!END**************************************************************************** */
{

  int32 i;
  char timecodeb[27];
  char *cptr;
  char julday[4] = {'\0'};

  if(PGS_TD_ASCIItime_AtoB(timecodea,timecodeb)!=PGS_S_SUCCESS)
    modsmf(MODIS_F_SDP_ERROR, "PGS_TD_ASCIItime_AtoB", "meta.c, getjulianday\0");

  cptr = &timecodeb[5];

  for(i=0;i<3;i++) {
    julday[i] = *cptr; 
    cptr++;
  }

  return(atoi(julday));

}


