#ifndef mods_h
#define mods_h
#include "mfhdf.h"
#include "PGS_TD.h"
#include "PGS_PC.h"
#include "PGS_MET.h"
#include "PGS_MODIS_37092.h"
#ifdef DEBUG_MALLOC
#include "/usr/include/dmalloc.h"
#endif
#define PLATFORMLUN       800550

#if defined IRIS4 || defined LINUX
#define chand chand_
#define csalbr csalbr_
#define sunglint sunglint_
#define readluts readluts_
#endif

#ifdef LINUX
#define sinf  sin
#define asinf asin
#define cosf  cos
#define powf  pow
#define expf  exp
#define logf  log
#define sqrtf sqrt
#define atan2f atan2
#define fabsf fabs
#endif

#define METAHANDLES       mdhandles
#define COREHANDLES       mdhandles[1]
#define ARCHIVEHANDLES    mdhandles[2]
#define PGEVERSIONNUMBERLUN 800500
#define SATURATED_VALUE 65533
#define DATAFILLVALUE   -28672

#define ATMOS_PER_MBAR (1.0F/1013.0F)
#define WATERVAPORDEFAULT 2.93F
#define OZONEDEFAULT 0.319F
#define SURFACEPRESSUREDEFAULT 1013.0F * ATMOS_PER_MBAR

#define PI      3.141592654F
#define PI_180  0.017453292519943295  /* PI/180.0 */
#define DEG2RAD (PI/180.0F)
#define RAD2DEG (180.0F/PI)

#define TWO_THIRDS 0.6666666666F
#define FOUR_THIRDS 1.3333333333F
#define RAD1     430000      /* new for 5.1.0, destriped 1KM */
#define RAD500   209101      /* new for 5.0.0, destriped HKM */
#define RAD250   700000
#define GEO      600000
#define CLD      422500
#define REF               209000
#define RCOARSE           209002
#define NPOLEFNAMELUN     209010
#define NONPOLEFNAMELUN   209011
#define SPOLEFNAMELUN     209012
#define OUTPUT_LUN        209961
#define OUTPUTMCFLUN      209962
#define DEGDEMFILE        209504 /* 1 x 1 degree gridded DEM-derived avg exp(z/8) database */
#define AOTR_REJ_LUN      209555 /*  --> global_AOTR_rejection_mask.hdf (27-MAR-03)   */

/* 31-OCT-05 */
enum{NOMODEL, SMKL, SMKH, DSST, URBP, URBC, MODELS};
#define TAURAY_LUTLUN     209400
#define GASCOEF_LUTLUN    209401
#define ANGLES_LUTLUN     209402
#define SMKL_RLUTLUN      209403
#define SMKL_TLUTLUN      209404
#define SMKL_ALUTLUN      209405

#define SMKH_RLUTLUN      209406
#define SMKH_TLUTLUN      209407
#define SMKH_ALUTLUN      209408
#define DUST_RLUTLUN      209409
#define DUST_TLUTLUN      209410
#define DUST_ALUTLUN      209411
#define URBP_RLUTLUN      209412
#define URBP_TLUTLUN	  209413
#define URBP_ALUTLUN      209414
#define URBC_RLUTLUN      209415
#define URBC_TLUTLUN	  209416
#define URBC_ALUTLUN      209417



#define QAINTFIRE     0x0800
#define QAINTCM       0x0400
#define QAINTSNOW     0x8000

#define TIMECODEBSIZE 27    /* to hold ASCII time code B strings */

#define MCF_SREF 209001
#define MCF_SREFC 209003
#define MCF_IDN 209007
#define MCF_IDT 209008
#define MCF_IDS 209009

#define OZONE_SCALE 0.001F

/* The DAO data set LUNs */
#define ANCWVT 900500 
#define SURFPRESSURE 900504 
#define OZONEFILE 900507  

/* The GRIB data set LUNs; new as of 16-JUN-99; updated, Oct, 2000 */
#define GRIB_ANCWVT 900000 
#define GRIB_SURFPRESSURE 900000 
#define GRIB_OZONEFILE 900020 
#define EPT_OZONEFILE 900060 

enum{RAD250_FILE, RAD500_FILE, RAD1_FILE, GEO_FILE, CLD_FILE, ALL_L1B_FILES};
typedef enum{REFPROD, IDSPROD, COARSEPROD}filetype_t;
#define LHDF_REF_MAX_SAMP "Data Samples Per Line"
#define LHDF_REF_DLINE    "Data Lines"

typedef enum{FINE_ONLY, COARSE_ONLY, FCBOTH, FCNEITHER} FC_output;
typedef enum{DAY_ONLY,  NIGHT_ONLY,  DNBOTH, DNNEITHER} DN_output;

enum{BAND1, BAND2, BAND3, BAND4, BAND5, BAND6, BAND7};

enum{BAND1_250M, BAND2_250M, BAND1_500M, BAND2_500M, BAND3_500M,
     BAND4_500M, BAND5_500M, BAND6_500M, BAND7_500M, BAND1_1KM, 
     BAND2_1KM,  BAND3_1KM,  BAND4_1KM,  BAND5_1KM,  BAND6_1KM, 
     BAND7_1KM,  BAND8,      /* New to 5.1.0 */
                             BAND9,      BAND10,     BAND11, 
     BAND12,     BAND13,     BAND14,     BAND15,     BAND16,    
     /* End of new stuff */
     
                             BAND18,     BAND19,     BAND20,  
     BAND21,     BAND22,     BAND23,     BAND26,     BAND29,     
     BAND31,     BAND32,   BANDPROCESS,  BAND20ALB,  AOT_1,      
     AOT_3,      AOT_8,      AOT_M,      SURFP,      WATERVAPOR, NCEPWATERVAPOR, OZONE,      
     AIR_TEMP,   LAT,        LON,        SOLZ, SATZ, RPHI, U_WND, V_WND, QKMQA,      HKMQA,      
     OKMQA,      STATE,      AOTCM,      AOTQA,      CRSQA,      
     CRSCM,
     ALLBANDS};

#define N_AOTQA_VALUES 19    /* Set this to the maximum value AOTQA can take: for IDS AOTQA handling... */

static char msg[PGS_SMF_MAX_MSG_SIZE]; /* buffer to hold SMF message */
/*PGSt_MET_all_handles METAHANDLES; / * ECS metadata handles * /   */

#define GRINGPOINTS 4
#define AOTR_SCALE 1000.0F
#define AOT1_FILL 60
#define AOT3_FILL 60
#define AOT8_FILL 0   /* new, 20-JUL-05: will put Angstrom exponents into "AOT8" */

#define NBANDS 16
#define NREFBN 16
#define NTS    22
#define NXX    20
#define NPR    7
#define NAOT   15

#define LANDLAND 0x1
#define LANDCOAST 0x2
#define EPHEMH2O 0x4   /* new, 13-APR-01: treat ephemeral water as land in band 20 albedo, internal CM */
#define LANDDO 0x7
#define LANDMSK 0x38
#define LANDCHECK(x) ((x & LANDMSK)!=(LANDDO<<3))
#define ACTUALLANDCHECK(x) ((x & LANDMSK)==(LANDLAND<<3) || (x & LANDMSK)==(LANDCOAST<<3) || (x & LANDMSK)==(EPHEMH2O<<3))


#define REFSCALE 10000.0F
#define REF_SLOPE (1.0F/REFSCALE)

#define A_PHI0 0.0F
#define A_DELPHI 2.5F
#define A_NBPHI 73
#define A_TS0 2.841F
#define A_DELTS 3.71F
#define A_NBTS 22
#define A_TV0 2.841F
#define A_DELTV 3.71F
#define A_NBTV 22
#define A_NBAOT 10
#define T_THETA0 0.0F
#define T_DELTHETA 5.0F
#define T_NBTHETA 17
#define T_NBAOT 10

#define BANDQA1(n, b) (n << ((b-9) * 4) + 2)
#define BANDQA500(n, b) (n << ((b-2) * 4) + 2)
#define BANDQA250(n, b) (n << (b * 4) + 4)
#define INVBANDQA1(n, b) (n >>((b-9) * 4) + 2)
#define INVBANDQA500(n, b) (n >> ((b-2) * 4) + 2)
#define INVBANDQA250(n, b) (n >> (b * 4) + 4)


#define REFMINVALUE -0.01F
/* REFMINQAVALUE must always be higher than REFMINVALUE */
#define REFMINQAVALUE 0.0F
#define REFMAXVALUE 1.6F
#define SCALEDREFMINVALUE -100
#define SCALEDREFMAXVALUE 10000
#define ATMCORR1 0x40000000
#define ATMCORR500 0x40000000
#define ATMCORR250 0x1000

#define LOWAEROSOL 0.0F
#define AVEAEROSOL 0.23F
#define HIGHAEROSOL 0.5F
#define RMR_AVEAEROSOL 0.015F
#define RMR_HIGHAEROSOL 0.030F
#define QALOWAEROSOL  0x0040
#define QAAVEAEROSOL  0x0080
#define QAHIGHAEROSOL 0x00c0
#define QACLIMATOLOGY 0x4000
#define CLOUDBYTES 6
#define IISCLOUDSET 0x1
#define IISKMCLOUD 0x6
#define IIS250MCLOUD 0x1
#define IISKMMIXEDA 0x2
#define IISKMMIXEDB 0x4
#define IISKMMIXED 0x6
#define IISKMSHADOW 0x4

#define OISCLOUD1KM 0x1
#define OISCLOUD250M 0x4
#define OISCLOUDNOTSET1KM 0x3
#define OISCLOUDNOTSET250M 0xc
#define MODLANDISCLOUD 0x2
#define ICM_CLOUDMASK   0x0001
#define ICM_CLDSMASK    0x0100
#define ICM_TEST(x,y) ((x) & (y))
#define ICM_FIREMASK    0x0020
#define CMASKSNOWICE 0x20

/*#define HIGHSOLARZENITH 75.0F*/
#define HIGHSOLARZENITH 85.0F
#define TERMINATORSZ 86.0F
/* per pixel qa values */
#define BADBAND 0xf
#define BADL1B 0xe
#define DEAD 0x8
#define BANDPOOROUTOFBOUNDS 0xd
#define USEDATMCONSTANT 0xc
#define MISSINGINPUTL1B 0xb
#define BADSOLARZENITH 0xa
#define TERMANGLE 0x9
/* L1B QA */
#define L1BPROBINDICATOR 0x8000
#define L1BMISSING 0xffff
#define OISCLOUD1KM 0x1
#define OISCLOUD250M 0x4
#define OISMIXED1KM 0x2
#define OISMIXED250M 0x8
#define OISSHADOW 0x4
#define OISCLOUDNOTSET1KM 0x3
#define OISCLOUDNOTSET250M 0xc
#define OISCLEARMIXTEST1KM 0x3
#define OISCLEARMIXTEST250M 0xc
#define OISCLEAR 0x0
#define OISBADPIXEL 0x3
#define OSNOWICE 0x1000
#define OISADJACENT 0x8000

#define SREFL_COARSESIZE 5

#define LGIDVERSIONNUMBER "004"

#ifdef MOD02CRS_PROC  /* 08-MAR-00 */
  #define NBSCANS_1KM	2
#else
  #define NBSCANS_1KM	10
#endif
#define NBSCANS_500M	20
#define NBSCANS_250M	40
#ifdef MOD02CRS_PROC  /* 08-MAR-00 */
  #define NBSPOTS_1KM	271
#else
  #define NBSPOTS_1KM	1354
#endif
#define NBSPOTS_500M	2708
#define NBSPOTS_250M	5416

#ifdef SUBSET
#define NBSPOTS_1KM	 50
#define NBSPOTS_500M	100
#define NBSPOTS_250M	200
#endif

#define XA_H2O_M1 -5.73888F
#define XB_H2O_M1 0.925534F
#define XC_H2O_M1 -0.0188365F
#define XA_H2O_M2 -5.3296F
#define XB_H2O_M2 0.824267F
#define XC_H2O_M2 -0.0277443F
#define XA_H2O_M3 0.0F
#define XB_H2O_M3 0.0F
#define XC_H2O_M3 0.0F
#define XA_H2O_M4 0.0F
#define XB_H2O_M4 0.0F
#define XC_H2O_M4 0.0F
#define XA_H2O_M5 -6.39296F
#define XB_H2O_M5 0.942186F
#define XC_H2O_M5 -0.0131901F
#define XA_H2O_M6 -7.76288F
#define XB_H2O_M6 0.979707F
#define XC_H2O_M6 0.0077841F
#define XA_H2O_M7 -4.05388F
#define XB_H2O_M7 0.872951F
#define XC_H2O_M7 -0.0268464F

#define XA_O3_M1 0.0715289F
#define XA_O3_M2 0.0F
#define XA_O3_M3 0.0074323F
#define XA_O3_M4 0.089691F
#define XA_O3_M5 0.0F
#define XA_O3_M6 0.0F
#define XA_O3_M7 0.0F

#define XA_CO2_M1 0.0F
#define XB_CO2_M1 0.0F
#define XC_CO2_M1 0.0F
#define XD_CO2_M1 0.0F
#define XA_CO2_M2 0.0F
#define XB_CO2_M2 0.0F
#define XC_CO2_M2 0.0F
#define XD_CO2_M2 0.0F
#define XA_CO2_M3 0.0F
#define XB_CO2_M3 0.0F
#define XC_CO2_M3 0.0F
#define XD_CO2_M3 0.0F
#define XA_CO2_M4 0.0F
#define XB_CO2_M4 0.0F
#define XC_CO2_M4 0.0F
#define XD_CO2_M4 0.0F
#define XA_CO2_M5 0.52977F
#define XB_CO2_M5 0.91705F
#define XC_CO2_M5 0.0002721F 
#define XD_CO2_M5 0.0000555F
#define XA_CO2_M6 0.5779F    
#define XB_CO2_M6 0.79394F
#define XC_CO2_M6 0.010412F  
#define XD_CO2_M6 0.0009856F
#define XA_CO2_M7 0.58906F      
#define XB_CO2_M7 0.7659F
#define XC_CO2_M7 0.0087812F 
#define XD_CO2_M7 0.0077764F

#define XA_O2_M1 0.61072F 
#define XB_O2_M1 0.71412F
#define XC_O2_M1 0.00093235F
#define XD_O2_M1 0.00007939F
#define XA_O2_M2 0.0F
#define XB_O2_M2 0.0F
#define XC_O2_M2 0.0F
#define XD_O2_M2 0.0F   
#define XA_O2_M3 0.0F
#define XB_O2_M3 0.0F
#define XC_O2_M3 0.0F
#define XD_O2_M3 0.0F
#define XA_O2_M4 0.0F
#define XB_O2_M4 0.0F
#define XC_O2_M4 0.0F
#define XD_O2_M4 0.0F
#define XA_O2_M5 0.61331F
#define XB_O2_M5 0.7087F 
#define XC_O2_M5 0.0016305F
#define XD_O2_M5 0.0004485F
#define XA_O2_M6 0.0F
#define XB_O2_M6 0.0F
#define XC_O2_M6 0.0F
#define XD_O2_M6 0.0F
#define XA_O2_M7 0.0F
#define XB_O2_M7 0.0F
#define XC_O2_M7 0.0F
#define XD_O2_M7 0.0F

#define XA_NO2_M1 0.0F
#define XB_NO2_M1 0.0F
#define XC_NO2_M1 0.0F
#define XD_NO2_M1 0.0F
#define XA_NO2_M2 0.0F
#define XB_NO2_M2 0.0F
#define XC_NO2_M2 0.0F
#define XD_NO2_M2 0.0F
#define XA_NO2_M3 0.0F
#define XB_NO2_M3 0.0F
#define XC_NO2_M3 0.0F
#define XD_NO2_M3 0.0F
#define XA_NO2_M4 0.0F
#define XB_NO2_M4 0.0F
#define XC_NO2_M4 0.0F
#define XD_NO2_M4 0.0F
#define XA_NO2_M5 0.0F
#define XB_NO2_M5 0.0F
#define XC_NO2_M5 0.0F
#define XD_NO2_M5 0.0F
#define XA_NO2_M6 0.0F
#define XB_NO2_M6 0.0F
#define XC_NO2_M6 0.0F
#define XD_NO2_M6 0.0F
#define XA_NO2_M7 0.51419F   
#define XB_NO2_M7 0.96079F
#define XC_NO2_M7 0.0040352F 
#define XD_NO2_M7 -0.00008074F

#define XA_CH4_M1 0.0F
#define XB_CH4_M1 0.0F
#define XC_CH4_M1 0.0F
#define XD_CH4_M1 0.0F
#define XA_CH4_M2 0.0F
#define XB_CH4_M2 0.0F
#define XC_CH4_M2 0.0F
#define XD_CH4_M2 0.0F
#define XA_CH4_M3 0.0F
#define XB_CH4_M3 0.0F
#define XC_CH4_M3 0.0F
#define XD_CH4_M3 0.0F
#define XA_CH4_M4 0.0F
#define XB_CH4_M4 0.0F
#define XC_CH4_M4 0.0F
#define XD_CH4_M4 0.0F
#define XA_CH4_M5 0.0F
#define XB_CH4_M5 0.0F
#define XC_CH4_M5 0.0F
#define XD_CH4_M5 0.0F
#define XA_CH4_M6 0.53852F  
#define XB_CH4_M6 0.89528F
#define XC_CH4_M6 0.0087631F
#define XD_CH4_M6 0.0009081F
#define XA_CH4_M7 0.5226F   
#define XB_CH4_M7 0.93679F
#define XC_CH4_M7 0.0003404F
#define XD_CH4_M7 0.0000364F

#define VRANGELOWREF -100
#define VRANGEHIGHREF 16000
#define FILLVALUEQUALITYU16 65535
#define VRANGEHIGHQUALITYU16 16383
#define VRANGELOWQUALITYU16 0
#define FILLVALUEQUALITYU32 3
#define VRANGEHIGHQUALITYU32 4294967295UL
#define VRANGELOWQUALITYU32 0
#define FILLVALUESTATE 65535
#define VRANGEHIGHSTATE 49151
#define VRANGELOWSTATE 0

#endif /* part of #ifndef/#define statements at beginning of file */
