#include "modis_srefl.h"
#include "protos.h"
#define MODLANDNOTPRODUCED 0x3
#define MODLANDPRODUCEDPOOR 0x1
extern int n_bands_res[3]; /* 250m, 500m, 1km */

void initqavalues(qavals_t *qavalues)
{
int i,j;

for(i=0;i<BANDPROCESS;i++) 
  for(j=0;j<16;j++) 
     qavalues->quality_count[i][j] = 0;
     

qavalues->qapercentgoodquality = 0L;
qavalues->qapercentother = 0L;
qavalues->qapercentnotproducedcloud = 0L;
qavalues->qapercentnotproducedother = 0L;
qavalues->qapercentoutofbounds = 0L;
qavalues->qapercentmissing = 0L;
qavalues->percentnotsea = 0L;
qavalues->atmcor_percentage = 0L;
qavalues->highsolarzenith = 0;
qavalues->highsolarzenith_tot = 0;
for(i=0;i<4;i++)
   qavalues->aerosol_clmh[i] = 0;
   
}


void count_quality(int32 *bands, dims_struct *output, 
		   qavals_t *qavalues, int32 process_sea, int32 process_clouds)
/*
!C*****************************************************************************
!Description:  Counts the per pixel qa values so that granule level metadata
               cam be created later

!Input Parameters:
bands -- array indicating which bands are being processed
sqc250m -- 250m qa data
sqc500m_quality -- 500m band qa data
sqc1km_quality -- 1km band qa data
sqc1km_state -- 1km earth state data

!Output Parameters: none

!Input/Output Parameters:
quality_count -- structure containing total number of pixels in a band quality
                 class per band processed.  Running total updated each 
		 data line.
qapercentgoodquality -- contains running total of the number of pixels that
                 are in the qa class PERCENTGOODQUALITY
qapercentother -- contains running total of the number of pixels that are in 
                 the qa class PERCENTOTHERQUALITY
qapercentnotproducedcloud -- contains running total of the number of pixels 
                 that are in the qa class PERCENTNOTPRODUCEDCLOUD
qapercentnotproducedother -- contains running total of the number of pixels
                 that are in the qa class PERCENTNOTPRODUCEDOTHER
qapercentoutofbounds -- contains running total of the number of pixels
                 that have their final result constrained within the valid
                 range
qapercentmissing -- contains running total of the number of pixels with
                  missing L1B input data.
smch -- output product correct surface reflectance data
percentnotsea -- contains running total of the number of pixels that are not 
                 deep ocean, deep inland water, or continental shelf ocean 
                 (i.e. the pixels processed because they are land or are 
                 considered to be equivalent to land from the standpoint of 
                 process/noprocess
          

!Returns: none

!Revision History:
        Modified, December 1998, P. Fisher
	1.  Updated QA logic
	2.  Calculated percent not sea using ACTUALLANDCHECK vs. LANDCHECK

	Original version 1997, P. Fisher (fisher@ltpmail.gsfc.nasa.gov).

!Team-unique Header:

   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits

		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980


!Design Notes
!END***************************************************************************
*/
{
  
  int32 b, r, c, r1, c1, index, index1km, index500m, index250m;
  unsigned long holder, value;
  enum{GOOD, CLOUD, NOTLAND, CLIMATOLOGY}pvalue;
  uint16 mv;
  /* new, 31-OCT-02 */
  int det;
  
  int16 *data;
  uint16 *sqc250m;
  uint32 *sqc500m_quality; 
  uint32 *sqc1km_quality;
  unsigned short *sqc1km_state; 
  
  sqc250m = output->data[QKMQA];
  sqc500m_quality = output->data[HKMQA];
  sqc1km_quality = output->data[OKMQA];
  sqc1km_state = output->data[STATE];

  if(n_bands_res[0] > 0){
    index250m=0;
    for (r=0;r<NBSCANS_250M;r++) {
    for (c=0;c<NBSPOTS_250M;c++) {
    /*for(index=0;index<NBSPOTS_250M * NBSCANS_250M;index++)*/

      r1 = r/4;
      c1 = c/4;
      index1km = r1*NBSPOTS_1KM + c1;
      
      /* state checks */
      pvalue=GOOD;
      if(!LANDCHECK(sqc1km_state[index1km]) && !process_sea)pvalue=NOTLAND;
      else if(( (sqc250m[index250m] & OISCLEARMIXTEST250M)==OISCLOUD250M) && !process_clouds )pvalue=CLOUD;
      /*else if(!(sqc1km_state[index1km] & QAMODISWV) && !(sqc1km_state[index1km] & QAMODISOZONE))
	pvalue=CLIMATOLOGY;*/

      /* set modland QA */
      if(pvalue==NOTLAND)sqc250m[index250m]|=MODLANDNOTPRODUCED;
      else if(pvalue==CLOUD)sqc250m[index250m]|=MODLANDISCLOUD;  
      else if(pvalue==CLIMATOLOGY)sqc250m[index250m]|=MODLANDPRODUCEDPOOR;

      if((mv=(sqc1km_state[index1km] & QAHIGHAEROSOL))==QAHIGHAEROSOL)qavalues->aerosol_clmh[3]++;
      else if(mv==QAAVEAEROSOL)qavalues->aerosol_clmh[2]++;
      else if(mv==QALOWAEROSOL)qavalues->aerosol_clmh[1]++;
      else qavalues->aerosol_clmh[0]++;

      if(!(sqc250m[index250m] & ATMCORR250) && pvalue!=CLOUD)sqc250m[index250m]|=MODLANDNOTPRODUCED;

      holder=sqc250m[index250m]>>4;

      /* read and set band quality */
      for(b=BAND1_250M;b<BAND1_500M;b++){
        data = output->data[b];
	value=holder & 0xf;
	if(bands[b]){
	  qavalues->quality_count[b][value]++;

	  /* recheck as NOTLAND is only NOTLAND for processing */
	  if(ACTUALLANDCHECK(sqc1km_state[index1km])) qavalues->percentnotsea++;

	  switch(value){
	  case 15: 
	    if(pvalue==NOTLAND)qavalues->qapercentnotproducedother++;
	    else if(pvalue==CLOUD)qavalues->qapercentnotproducedcloud++;
	    else if(data[index250m]==DATAFILLVALUE){
	      qavalues->qapercentmissing++;
	      sqc250m[index250m]|=MODLANDNOTPRODUCED;
	    } 
	    break;
	  case 14:
	    qavalues->qapercentnotproducedother++;
	    sqc250m[index250m]|=MODLANDNOTPRODUCED;
	    break;
	  case 13:
	    qavalues->qapercentoutofbounds++;
	    if(!(sqc250m[index250m] & 0x3))sqc250m[index250m]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 12:
	    qavalues->qapercentother++;
	    if(!(sqc250m[index250m] & 0x3))sqc250m[index250m]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 11:qavalues->qapercentnotproducedother++;
	    sqc250m[index250m]|=MODLANDNOTPRODUCED;
	    break;
	  case 10:qavalues->qapercentnotproducedother++;
	    sqc250m[index250m]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 9:qavalues->qapercentnotproducedother++;
	    sqc250m[index250m]|=MODLANDNOTPRODUCED;
	    break;
	  case 0: qavalues->qapercentgoodquality++;
	    break;
	  case 8: /* 01-MAY-00, JPR: I have yet to decide what to do with dead detector QA */
                  qavalues->qapercentother++;  /* for now do this */
	    break;
	  default:
	    modsmf(MODIS_E_UNCODED_QA, "", "qa.c, count_quality\0");
	    break;
	  }
	}
	holder=holder>>4;

	
	if(bands[b])if(sqc250m[index250m] & ATMCORR250)qavalues->atmcor_percentage++;

      }/* for(b=BAND1_250M;b<BAND1_500M;b++) */   
       	
      index250m++;
    }
    }/* for(index=0;index<NBSPOTS_250M * NBSCANS_250M;index++) */
  }/* if(bands[BANDS250]) */

  if(n_bands_res[1] > 0){
    index500m = 0;
    for (r=0;r<NBSCANS_500M;r++) {
    for (c=0;c<NBSPOTS_500M;c++) {
    /*for(index=0;index<NBSPOTS_500M * NBSCANS_500M;index++)*/
      
      r1 = r/2;
      c1 = c/2;
      index1km = r1*NBSPOTS_1KM + c1;

      /* state checks */
      pvalue=GOOD;
      if(!LANDCHECK(sqc1km_state[index1km]) && !process_sea)pvalue=NOTLAND;
      else if(((sqc1km_state[index1km] & OISCLEARMIXTEST1KM)==OISCLOUD1KM) && !process_clouds)pvalue=CLOUD;
      /*else if(!(sqc1km_state[index1km] & QAMODISWV) && !(sqc1km_state[index1km] & QAMODISOZONE))
	pvalue=CLIMATOLOGY;*/
      
      /* set modland QA */
      if(pvalue==NOTLAND)sqc500m_quality[index500m]|=MODLANDNOTPRODUCED;
      else if(pvalue==CLOUD)sqc500m_quality[index500m]|=MODLANDISCLOUD;  
      else if(pvalue==CLIMATOLOGY)sqc500m_quality[index500m]|=MODLANDPRODUCEDPOOR;

      if((mv=(sqc1km_state[index1km] & QAHIGHAEROSOL))==QAHIGHAEROSOL)qavalues->aerosol_clmh[3]++;
      else if(mv==QAAVEAEROSOL)qavalues->aerosol_clmh[2]++;
      else if(mv==QALOWAEROSOL)qavalues->aerosol_clmh[1]++;
      else qavalues->aerosol_clmh[0]++;

      if(!(sqc500m_quality[index500m] & ATMCORR500) && pvalue!=CLOUD)sqc500m_quality[index500m]|=MODLANDNOTPRODUCED;

      holder=sqc500m_quality[index500m]>>2;
      
      /* 31-OCT-02 */
      det = index500m/NBSPOTS_500M;

      /* read and set band quality */
      for(b=BAND1_500M;b<BAND1_1KM;b++){
        data = output->data[b];
	value=holder & 0xf;
	if(bands[b]){
	  qavalues->quality_count[b][value]++;

	  /* recheck as NOTLAND is only NOTLAND for processing */
	  if(ACTUALLANDCHECK(sqc1km_state[index1km]))qavalues->percentnotsea++;
	
	  switch(value){
	  case 15: 
	    if(pvalue==NOTLAND)qavalues->qapercentnotproducedother++;
	    else if(pvalue==CLOUD)qavalues->qapercentnotproducedcloud++;
	     else if(data[index500m]==DATAFILLVALUE){
	      qavalues->qapercentnotproducedother++;
	      qavalues->qapercentmissing++;
	      sqc500m_quality[index500m]|=MODLANDNOTPRODUCED;
	    } 	    
	    break;
	  case 14:
	    if ((b == BAND5_500M) && (det == 3)) {  /* 31-OCT-02: ASSUMING *ALL* case 14's for band5/det 4 are
	                                                put there by ME on 31-OCT-02, at request of LDOPE, and 
							do NOT reflect actual faulty (negative) L1B. */
	       qavalues->qapercentother++;
	       if(!(sqc500m_quality[index500m] & 0x3))sqc500m_quality[index500m]|=MODLANDPRODUCEDPOOR;
	    }
	    else {
	       qavalues->qapercentnotproducedother++;
	       sqc500m_quality[index500m]|=MODLANDNOTPRODUCED;
	    }
	    break;
	  case 13:
	    qavalues->qapercentoutofbounds++;
	    if(!(sqc500m_quality[index500m] & 0x3))sqc500m_quality[index500m]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 12:
	    qavalues->qapercentother++;
	    if(!(sqc500m_quality[index500m] & 0x3))sqc500m_quality[index500m]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 11:
	    qavalues->qapercentnotproducedother++;
	    sqc500m_quality[index500m]|=MODLANDNOTPRODUCED;
	    break;
	  case 10:
	    qavalues->qapercentnotproducedother++;
	    sqc500m_quality[index500m]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 9:
	    qavalues->qapercentnotproducedother++;
	    sqc500m_quality[index500m]|=MODLANDNOTPRODUCED;
	    break;
	  case 0: 
	    qavalues->qapercentgoodquality++;
	    break;
	  case 8: /* 01-MAY-00, JPR: I have yet to decide what to do with dead detector QA */
                  qavalues->qapercentother++;  /* for now do this */
	    break;
	  default:
	    modsmf(MODIS_E_UNCODED_QA, "", "qa.c, count_quality\0");
	    break;
	  }
	}
	holder=holder>>4;
	
	if(bands[b])if(sqc500m_quality[index500m] & ATMCORR500)qavalues->atmcor_percentage++;

      }/* for(b=BAND1_500M;b<BAND1_1KM;b++) */

      index500m++;
      }
    }/* for(index=0;index<NBSPOTS_500M * NBSCANS_500M;index++) */
  }/* if(bands[BANDS500]) */

  if(n_bands_res[2] > 0){
    for(index=0;index<NBSPOTS_1KM * NBSCANS_1KM;index++){
      
      /* state checks */
      pvalue=GOOD;
      if(!LANDCHECK(sqc1km_state[index]) && !process_sea)pvalue=NOTLAND;
      else if(((sqc1km_state[index] & OISCLEARMIXTEST1KM)==OISCLOUD1KM) && !process_clouds)pvalue=CLOUD;
      /*else if(!(sqc1km_state[index] & QAMODISWV) && !(sqc1km_state[index] & QAMODISOZONE))
	pvalue=CLIMATOLOGY;*/
      
      /* set modland QA */
      if(pvalue==NOTLAND)sqc1km_quality[index]|=MODLANDNOTPRODUCED;
      else if(pvalue==CLOUD)sqc1km_quality[index]|=MODLANDISCLOUD;  
      else if(pvalue==CLIMATOLOGY)sqc1km_quality[index]|=MODLANDPRODUCEDPOOR;

      if((mv=(sqc1km_state[index] & QAHIGHAEROSOL))==QAHIGHAEROSOL)qavalues->aerosol_clmh[3]++;
      else if(mv==QAAVEAEROSOL)qavalues->aerosol_clmh[2]++;
      else if(mv==QALOWAEROSOL)qavalues->aerosol_clmh[1]++;
      else qavalues->aerosol_clmh[0]++;

      if(!(sqc1km_quality[index] & ATMCORR500) && pvalue!=CLOUD)sqc1km_quality[index]|=MODLANDNOTPRODUCED;

      holder=sqc1km_quality[index]>>2;

      /* read and set band quality */
      for(b=BAND1_1KM;b<BAND8;b++){
        data = output->data[b];
	value=holder & 0xf;
	if(bands[b]){
	  qavalues->quality_count[b][value]++;

	  /* recheck as NOTLAND is only NOTLAND for processing */
	  if(ACTUALLANDCHECK(sqc1km_state[index]))qavalues->percentnotsea++;

	  switch(value){
	  case 15: 
	    if(pvalue==NOTLAND)qavalues->qapercentnotproducedother++;
	    else if(pvalue==CLOUD)qavalues->qapercentnotproducedcloud++;
	    else if(data[index]==DATAFILLVALUE){
	      qavalues->qapercentmissing++;
	      sqc1km_quality[index]|=MODLANDNOTPRODUCED;
	      } 
	    break;
	  case 14:
	    qavalues->qapercentnotproducedother++;
	    sqc1km_quality[index]|=MODLANDNOTPRODUCED;
	    break;
	  case 13:
	    qavalues->qapercentoutofbounds++;
	    if(!(sqc1km_quality[index] & 0x3))sqc1km_quality[index]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 12:
	    qavalues->qapercentother++;
	    if(!(sqc1km_quality[index] & 0x3))sqc1km_quality[index]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 11:
	    qavalues->qapercentnotproducedother++;
	    sqc1km_quality[index]|=MODLANDNOTPRODUCED;
	    break;
	  case 10:
	    qavalues->qapercentnotproducedother++;
	    sqc1km_quality[index]|=MODLANDPRODUCEDPOOR;
	    break;
	  case 9:
	    qavalues->qapercentnotproducedother++;
	    sqc1km_quality[index]|=MODLANDNOTPRODUCED;
	    break;
	  case 0: 
	    qavalues->qapercentgoodquality++;
	    break;
	  case 8: /* 01-MAY-00, JPR: I have yet to decide what to do with dead detector QA */
                  qavalues->qapercentother++;  /* for now do this */
	    break;
	  default:
	    modsmf(MODIS_E_UNCODED_QA, "", "qa.c, count_quality\0");
	    break;
	  }
	}
	holder=holder>>4;
	
	if(bands[b])if(sqc1km_quality[index] & ATMCORR1)qavalues->atmcor_percentage++;

      }/* for(b=BAND1_1KM;b<BANDPROCESS;b++) */

    }/* for(index=0;index<NBSPOTS_1KM * NBSCANS_1KM;index++) */
  }/* if(bands[BANDS1]) */

  return;
}
