#ifndef strt_h
#define strt_h
typedef long int LINT;
typedef struct{
  float32 *data;
  int32 dims[3];
  int32 wasread;
  float32 latres;
  float32 lonres;
  int numdays;
  int lasttimeplane;
  int lastversion[2];
  /* 16-JUN-99: structure d_ancillary will be used to handle either DAO *OR*
     GRIB-format data; anci_source will be either "GRIB" or "DAO", tag will be
     either GRIB or DAO-specific tags, what to look for in the file.
     Jim Ray, SSAI */
  char anci_source[5];
  char tag[10];
  float32 fillvalue;
  LINT jrbd[2];
  int versions[2];
  float weights[2];
} d_ancillary;

typedef struct {
  float32 xaH2O[7],xbH2O[7],xcH2O[7];
  float32 xaO3[7];
  float32 xaCO2[7],xbCO2[7];
  float32 xcCO2[7],xdCO2[7];
  float32 xaO2[7],xbO2[7];
  float32 xcO2[7],xdO2[7];
  float32 xaNO2[7],xbNO2[7];
  float32 xcNO2[7],xdNO2[7];
  float32 xaCH4[7],xbCH4[7];
  float32 xcCH4[7],xdCH4[7];
  float32 tau_r[7];
} t_atmospars;

/* Structure used to track ECS metadata array strings */ 
typedef struct{
  char **marray;
  int32 arcounter;
} marray_t;

typedef enum{AUTOBAD, AUTOGOOD, AUTOSUSPECT} aq_e;

/* metadata structure */
typedef struct{  
  marray_t *inputpointer;
  marray_t *systemfilename;
  marray_t *ancinputpointer;
  marray_t *ancsystemfilename; 
  int32 *last_filename;
  int32 last_filename_index;
  char rbdt[PGSd_PC_VALUE_LENGTH_MAX];  /* RANGEBEGINNINGDATETIME */
  char redt[PGSd_PC_VALUE_LENGTH_MAX];  /* RANGEBEGINNINGDATETIME */
  int32 orbitnumber;
  int32 granulenumber;
  char daynightflag[PGSd_PC_VALUE_LENGTH_MAX];
  float64 northboundingcoordinate;
  float64 southboundingcoordinate;
  float64 westboundingcoordinate;
  float64 eastboundingcoordinate;
  float64 gringpointlatitude[GRINGPOINTS];
  float64 gringpointlongitude[GRINGPOINTS];
  int32 gringpointsequenceno[GRINGPOINTS];
  char exclusiongringflag[PGSd_PC_VALUE_LENGTH_MAX];
  int32 instrumentscansprocessed;
  int32 sci_state;
  int32 sci_abnorm;
  char reprocessingplanned[PGSd_PC_VALUE_LENGTH_MAX];
  char reprocessingactual[PGSd_PC_VALUE_LENGTH_MAX];
  char seaprocessed[PGSd_PC_VALUE_LENGTH_MAX];
  char adjacencycorrectionattempted[PGSd_PC_VALUE_LENGTH_MAX];
  char atmosphericcorrectionattempted[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentgoodquality[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentotherquality[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentnotproducedcloud[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentnotproducedother[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput250mband1[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput250mband2[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband1[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband2[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband3[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband4[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband5[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband6[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput500mband7[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband1[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband2[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband3[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband4[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband5[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband6[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenameqapercentpooroutput1kmband7[PGSd_PC_VALUE_LENGTH_MAX];
  char attributenamegranulenumber[PGSd_PC_VALUE_LENGTH_MAX];
  char brdfcorrectionattempted[PGSd_PC_VALUE_LENGTH_MAX];
  int32 qualityclasspercentage[BANDPROCESS][16];
  int32 qapercentgoodquality;
  int32 qapercentother;
  int32 qapercentnotproducedcloud;
  int32 qapercentnotproducedother;
  int32 qapercentoutofbounds;
  int32 qapercentmissing;
  int32 percentnotsea;
  int32 percentland;
  char localgranuleidversion[PGSd_PC_VALUE_LENGTH_MAX];
  char localgranuleidversionfire[PGSd_PC_VALUE_LENGTH_MAX];
  char localgranuleid[PGSd_PC_VALUE_LENGTH_MAX];
  char localgranuleidfire[PGSd_PC_VALUE_LENGTH_MAX];
  char localgranuleidcoarse[PGSd_PC_VALUE_LENGTH_MAX];
  char pgeversion[PGSd_PC_VALUE_LENGTH_MAX];
  char autoquality[PGSd_PC_VALUE_LENGTH_MAX];
  char equatorcrossingdate[PGSd_PC_VALUE_LENGTH_MAX];
  char equatorcrossingtime[PGSd_PC_VALUE_LENGTH_MAX];
  float64 equatorcrossinglongitude;
  int32 highsolarzenith;
  PGSt_integer aerosol_clmh[4];
  char attributenameprocessversion[PGSd_PC_VALUE_LENGTH_MAX];
  char processversion[PGSd_PC_VALUE_LENGTH_MAX];
  /* included for efficiency */
  aq_e AutoQualityIsGood; 
  char used_int_cloud_mask[4];
  PGSt_double rbdt_tai;
  PGSt_double redt_tai;
  int32 int_pgeversion;
} input_metadata;
  
typedef struct {
  void *data[ALLBANDS];
  char *filename;
  int OK;  /* -1 = uninit, 0 = OK, 1 = error */
  int32 nblayers,nblines,nbcols;
  int32 ver, n_sds;
  int32 sds_offset[ALLBANDS];     /* for 3D SDSs, the number of the layer the parameter is found in */
  int32 band_name[ALLBANDS];      /* for 3D SDSs, the actual identity of that layer */
  float32 radscale[ALLBANDS], radoffset[ALLBANDS];
  float32 refscale[ALLBANDS], refoffset[ALLBANDS];
  int32 lun, sdid, sdsid[ALLBANDS];
  int32 type[ALLBANDS];
  input_metadata *xxxxx;
} dims_struct;

typedef struct{
  int32 adja_on;
  int32 brdf_mode;
  int32 coarse_product;
  int32 fire_product;
  int32 ref_product;
  int32 ref_lam;
  int32 ref_lamc;    /* New, 19-FEB-99 */
  int32 aot_retr;    /* New, 29-DEC-00, puts code into AOT retrieval mode */
  int32 process_sea;
  int32 atool;
  int32 atoolscan;
  int32 atoolspot;
  int32 make_cloud_shadow;  /* new, 24-MAR-04 */
  int32 use_modaot;
  int32 pol_corr;   /* new, 19-APR-01: polarization correction */
  int32 makegcm;    /* new, 27-FEB-02: GCM output */
} commands_t;

typedef struct {
  FC_output FC;
  DN_output DN;  
  int32 type;
  int16 lowrange, hirange, fillvalue;
  float64 scale;
  char units[20];
} output_type_t;

/* for AOT retrieval rejection */
typedef struct {
  uint32 *aotr_offsets;
  uint32 *aotr_lengths;
  int    n_aotr_offsets;
  float  aotr_resolution;
} aerosol_retr_rej_t;

/* 21-FEB-02: structure for CMG data output */
/* 24-JUL-02: new, for improved internal cloud mask processing */
typedef struct{
  int8 cloud, clear, ihc, lwc, snow, fire, glint, dust, cldshd, cldedge, cirrus, panflag;
}d_cmask;
/* 07-JUN-04: new, for AOT QA processing */
typedef struct{
  int8 aqa_values[N_AOTQA_VALUES+1];
}d_aotqa;
typedef struct{
  d_cmask *cmask;
  d_aotqa *aotqa;
  void *data;
  float32 *fdata;
  char SDSname[256];
  int32 dims[2];
  int32 sdsid[3];
  int32 sdid[3];
  int32 type;
  int currentsize;
} d_sds;

typedef struct{
  int32 quality_count[BANDPROCESS][16];
  long int qapercentgoodquality;
  long int qapercentother;
  long int qapercentnotproducedcloud;
  long int qapercentnotproducedother;
  long int qapercentoutofbounds;
  long int qapercentmissing;
  long int percentnotsea;
  long int atmcor_percentage;
  int32 highsolarzenith;
  int32 highsolarzenith_tot;
  PGSt_integer aerosol_clmh[4];
} qavals_t;


#endif /* part of #ifndef/#define statements at beginning of file */ 

