/*
!C******************************************************************

!Description: 
		utils.c


!Input Parameters:
!Output Parameters: 
!Revision History:
!Team-unique Header:
!References and Credits 
!Design Notes


DESCRIPTION

!END*************************************************************************/

/* NODAAC may need to be defined when running this code
in any non-DAAC processing mode.  This feature was added
by MODIS SDST to ensure compliance with ESDIS standards upon
delivering this software to the DAAC. */

#ifdef NODAAC
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef DEBUG_MALLOC
#include "/usr/include/dmalloc.h"
#endif

#include "smf.h"

/*
Do not define this for SDST or DAAC delivery!
For testing and debugging only (mainly by the SCF).
*/
#undef SCF


#include "PGS_IO.h"
#include "PGS_SMF.h"

/* PGS seed files */

#include "PGS_MODIS_37092.h"


#ifndef SCF
static char errstr[128];
#endif

void *ReAllocMem(void *ptr, size_t bytes)
/*
!C*****************************************************************************
!Description:
	Reallocate block of memory of specified size.  Issue error message and
	halt entire program if memory can not be allocated.

!Input Parameters:
        ptr             pointer to be reallocated
	bytes		Size of block in bytes.

!Output Parameters:
	Address of memory.

!Revision History:
        Version 1.1 Apr. 1997, P. Fisher/SSAI/GSFC Code 923
	Modified AllocMem to use realloc() instead of malloc().

	Original version Jan. 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
   This software developed by the MODIS Land Science Team for 
   the National Aeronautics and Space Administration, Goddard 
   Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Christopher O. Justice, justice@kratmos.gsfc.nasa.gov, 301-286-7372
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

		Louis Giglio
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
		giglio@hades.gsfc.nasa.gov, (301) 286-2091

!END***************************************************************************
*/
{

	if ( !(ptr = realloc(ptr, bytes)) ) {
#ifdef SCF
		Epro_printf("Error reallocating %d bytes.\n", bytes);
#else
		(void) sprintf(errstr, "%d bytes", bytes);
		modsmf(MODIS_F_MEM_FAILURE, errstr, "utils.c, ReAllocMem\0");
#endif
		exit(1);
		}

	return(ptr);
}

void *CAllocMem(size_t num, size_t size)
/*
!C*****************************************************************************
!Description:
	Allocate block of memory of specified size.  Issue error message and
	halt entire program if memory can not be allocated.

!Input Parameters:
	bytes		Size of block in bytes.

!Output Parameters:
	Address of memory.

!Revision History:
        Version 1.1 Mar. 1997, P. Fisher/SSAI/GSFC Code 923
	Modified AllocMem to use calloc() instead of malloc().

	Original version Jan. 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
   This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBERS
			Eric F Vermote, eric@kratmos.gsfc.nasa.gov, 301-286-6232
			Christopher O. Justice, justice@kratmos.gsfc.nasa.gov, 301-286-7372
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

		Louis Giglio
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
		giglio@hades.gsfc.nasa.gov, (301) 286-2091


!Design Notes:

!END***************************************************************************
*/
{
	void *vptr;

	if ( !(vptr = calloc(num, size)) ) {
#ifdef SCF
		Epro_printf("Error allocating %d bytes.\n", (int32)(num * size));
#else
		(void) sprintf(errstr, "%d bytes", (int32)num*size);
		modsmf(MODIS_F_MEM_FAILURE, errstr, "utils.c, CAllocMem\0");
#endif
		exit(1);
		}

	return vptr;
}

void *AllocMem(size_t bytes, char *calling_routine)
/*
!C*****************************************************************************
!Description:
	Allocate block of memory of specified size.  Issue error message and
	halt entire program if memory can not be allocated.

!Input Parameters:
	bytes		Size of block in bytes.

!Output Parameters:
	Address of memory.

!Revision History:
	Original version Jan. 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
        References and Credits

        Design Notes
	Obviously, this function should only be used to allocate memory that
	is absolutely essential for further processing.
!END***************************************************************************
*/
{
	void *vptr=NULL;

	if ( !(vptr = malloc(bytes)) ) {
#ifdef SCF
		Epro_printf("Error allocating %d bytes.\n", bytes);
#else
		(void) sprintf(errstr, "%d bytes, calling routine %s", bytes, calling_routine);
		modsmf(MODIS_F_MEM_FAILURE, errstr, "utils.c, AllocMem\0");
#endif
		exit(1);
		}

	return vptr;
}


PGSt_IO_Gen_FileHandle *OpenFileRead(PGSt_PC_Logical logical, PGSt_integer version)
/*
!C*****************************************************************************
!Description:
	Open file for reading given PGS logical file number.
	Issue error message and halt entire program if an error
	occurs.

!Input Parameters:
	logical		PGS logical file number.
	version		PGS file version (whatever that is).

!Output Parameters:
	ANSI C file pointer (PGSt_IO_Gen_FileHandle *);

!Revision History:
        Revision 1.1: May 1997, P. Fisher (SSAI/NASA's GSFC Code 923)
	1.  Function now returns (PGSt_IO_Gen_FileHandle *).  Prior to
	    this the function returned (FILE *) where the file handle
	    was really a cast of (PGSt_IO_Gen_FileHandle *).  Under unknown
	    circumstances, this may not be portable.

	Original version Jan. 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
        References and Credits

        Design Notes
	Obviously, this function should only be used to Epro_open those files that
	are absolutely essential for further processing.
!END***************************************************************************
*/

{
	PGSt_IO_Gen_AccessType Epro_access = PGSd_IO_Gen_Read;
	PGSt_IO_Gen_FileHandle *PGS_fp;
	PGSt_SMF_status st;

	
	if(PGS_IO_Gen_Open(logical, Epro_access, &PGS_fp, version)!=PGS_S_SUCCESS){
	  sprintf(errstr, "logical file ID %u", logical);
	  modsmf(MODIS_F_OPEN_GEN_FILE, errstr, "utils.c, OpenFileRead\0");
	}

	return (PGS_fp);
}

PGSt_IO_Gen_FileHandle *OpenFileRead_nf(PGSt_PC_Logical logical, PGSt_integer version)
/*
!C*****************************************************************************
!Description:
	Open file for reading given PGS logical file number.
	Issue error message if an error
	occurs.

!Input Parameters:
	logical		PGS logical file number.
	version		PGS file version (whatever that is).

!Output Parameters:
	ANSI C file pointer (PGSt_IO_Gen_FileHandle *);

!Revision History:
        Revision 1.2 May 1997, P. Fisher (SSAI/NASA's GSFC Code 923)
	1.  Modification of OpenFileRead() in order to provide non-fatal
	file handling.

        Revision 1.1: May 1997, P. Fisher (SSAI/NASA's GSFC Code 923)
	1.  Function now returns (PGSt_IO_Gen_FileHandle *).  Prior to
	    this the function returned (FILE *) where the file handle
	    was really a cast of (PGSt_IO_Gen_FileHandle *).  Under unknown
	    circumstances, this may not be portable.

	Original version Jan. 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
        References and Credits

        Design Notes
	Obviously, this function should only be used to Epro_open those files that
	are absolutely essential for further processing.
!END***************************************************************************
*/

{
	PGSt_IO_Gen_AccessType Epro_access = PGSd_IO_Gen_Read;
	PGSt_IO_Gen_FileHandle *PGS_fp;
	PGSt_SMF_status st;

	if(PGS_IO_Gen_Open(logical, Epro_access, &PGS_fp, version)!=PGS_S_SUCCESS){
	  sprintf(errstr, "logical file ID %u", logical);
	  modsmf(MODIS_E_OPEN_GEN_FILE, errstr, "utils.c, OpenFileRead_nf\0");
	}

	return (PGS_fp);
}



int32 CloseFile(PGSt_IO_Gen_FileHandle *fp)
/*
!C*****************************************************************************
!Description:
	Epro_close file given PGS logical file number.
	Issue error message if an error occurs.

!Input Parameters:
	fp		ANSI C file pointer of Epro_open file.

!Output Parameters:
	0 if no error closing file, non-zero if error.

!Returns: Always returns zero.

!Revision History:
        Version 1.1, May 1997, P. Fisher, SSAI/NASA's GSFC Code 923
	1.  Simplified code.
	2.  Changed error confition to _E_ from _F_, since the failure to
	    close a non HDF file will not necessarily be fatal to the process.
            All files will get closed by the OS anyway.

	Original version Jan. 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
        References and Credits
        Design Notes
!END***************************************************************************
*/

{
  PGSt_IO_Gen_FileHandle *PGS_fp;
  
  PGS_fp = (PGSt_IO_Gen_FileHandle *) fp;
  
  if (PGS_IO_Gen_Close(PGS_fp) != PGS_S_SUCCESS) {
    
    (void) sprintf(errstr, "\0");
    modsmf(MODIS_E_CLOSE_GEN_FILE, errstr, "utils.c, CloseFile\0");    
  }
  
  return 0;
}




int32 FileExists(PGSt_PC_Logical logical, PGSt_integer version)
/*
!C*****************************************************************************
!Description:
	Test whether a particular file exists for reading.

!Input Parameters:
	logical		PGS logical file number.
	version		PGS file version.

!Output Parameters:
	0 if file does NOT exist for reading, non-zero if file exists for
	reading.

!Revision History:
	Original version March 1996, L. Giglio (giglio@hades.gsfc.nasa.gov).

!Team-unique Header:
        References and Credits
        Design Notes
!END***************************************************************************
*/
{
	PGSt_IO_Gen_AccessType Epro_access = PGSd_IO_Gen_Read;
	PGSt_IO_Gen_FileHandle *PGS_fp;
	PGSt_SMF_status st;

	st = PGS_IO_Gen_Open(logical, Epro_access, &PGS_fp, version);
	if (st != PGS_S_SUCCESS)
		return 0;
	else {
		(void) PGS_IO_Gen_Close(PGS_fp);
		return -1;
		}
}




int32 band_to_spectral_index(int32 x)
/*
!C*****************************************************************************
!Description:

   Takes band sequence index (0-15) and returns spectral band index (0-6);

!Input Parameters:

!Output Parameters:

!returns:

!Revision History:

!Team-unique Header:

    This software developed by the MODIS Land Science Team for 
    the National Aeronautics and Space Administration, Goddard 
    Space Flight Center, under contract NAS5-96062

!References and Credits 

MODIS TEAM MEMBER
			Eric F Vermote
			Univerisity of MD.
		   	NASA's GSFC Code 923
		   	Greenbelt, MD , 20771
			eric@kratmos.gsfc.nasa.gov
			301-286-6232

DEVELOPERS:
		Paul Fisher
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		fisher@ltpmail.gsfc.nasa.gov, (301) 286-2980

		Louis Giglio
		Science Systems and Applications Inc.
		NASA's Goddard Space Flight Center Code 923
		Greenbelt MD, 20771
 		giglio@hades.gsfc.nasa.gov, (301) 286-2091

!Design Notes:

!END***************************************************************************
*/
{

  switch(x){
  case(0):
    return(0);
  case(1):
    return(1);
  case(2):
    return(0);
  case(3):
    return(1);
  case(4):
    return(2);
  case(5):
    return(3);
  case(6):
    return(4);
  case(7):
    return(5);
  case(8):
    return(6);
  case(9):
    return(0);
  case(10):
    return(1);
  case(11):
    return(2);
  case(12):
    return(3);
  case(13):
    return(4);
  case(14):
    return(5);
  case(15):
    return(6);
  default:
    modsmf(MODIS_F_BAD_BAND_SELECTION, NULL, "utils.c, band_to_spectral_index\0");
  }
  /* the funtion never returns -1 */
  return(-1);
}


PGSt_IO_Gen_FileHandle *OpenFileWrite(PGSt_PC_Logical logical, PGSt_integer version)
/*
!C*****************************************************************************
!Description:
	Open file for writing to a given PGS logical file number.
	Issue error message if an error
	occurs.

!Input Parameters:
	logical		PGS logical file number.
	version		PGS file version (whatever that is).

!Output Parameters:
	ANSI C file pointer (PGSt_IO_Gen_FileHandle *);

!Revision History:
	Original version Oct. 1998, P. Fisher

!Team-unique Header:
        References and Credits

!END***************************************************************************
*/

{
  PGSt_IO_Gen_AccessType Epro_access = PGSd_IO_Gen_Write;
  PGSt_IO_Gen_FileHandle *PGS_fp;
  
  
  if(PGS_IO_Gen_Open(logical, Epro_access, &PGS_fp, version)!=PGS_S_SUCCESS){
    sprintf(errstr, "logical file ID %u", logical);
    modsmf(MODIS_E_OPEN_GEN_FILE, errstr, "utils.c, OpenFileWrite\0");
  }
  
  return (PGS_fp);
}
